function [einList,integrand] = einsteinSingleParticleNumeric(dimList)
% This function numerically integrates the periodic potential for a single
% particle. output from this function for dimList = 3 is used in figure7.m

a = 1;
lambdaMin = 1e-4;
lambdaMax = 1e4;
k = 2*pi;
einList = logspace(log10(lambdaMin),log10(lambdaMax),100);
for dim = dimList
    % Set cell resolution dx for numerical integration. Lower values
    % produce more accurate results.
    if dim == 1
        dx = 0.0001;
    elseif dim == 2
        dx = 0.001;
    else
        dx = 0.001;
    end
    spacing = (dx/2):dx:a;
    integrand = zeros(size(einList));
    msdMean = zeros(size(einList));
    for l = 1:numel(einList)
        disp(l)
        einLam = einList(l);
        integralTotalTop = 0;
        integralTotalBottom = 0;
        if dim == 4
            for i = 1:numel(spacing)
                x = spacing(i);
                for j = 1:numel(spacing)
                    y = spacing(j);  
                    for k = 1:numel(spacing)
                        z = spacing(k);
                        for ii = 1:numel(spacing)
                            w = spacing(ii);
                            energy = (1 - cos(k*x)*cos(k*y)*cos(k*z)*cos(k*w))/2;
                            integralTotalTop = integralTotalTop + (energy*exp(-einLam*energy));
                            integralTotalBottom = integralTotalBottom + exp(-einLam*energy);
                        end
                    end
                end
            end
        elseif dim == 3
            for i = 1:numel(spacing)
                cx = cos(k*spacing(i));
                for j = 1:numel(spacing)
                    cy = cos(k*spacing(j))*cx;
                    for k = 1:numel(spacing)
                        energy = (1 - cy*cos(k*spacing(k)))/2;
                        integralTotalTop = integralTotalTop + (energy*exp(-einLam*energy));
                        integralTotalBottom = integralTotalBottom + exp(-einLam*energy);
                    end
                end
            end
        elseif dim == 2
            for i = 1:numel(spacing)
                x = spacing(i);
                for j = 1:numel(spacing)
                    y = spacing(j);            
                    energy = (1 - cos(k*x)*cos(k*y))/2;
                    integralTotalTop = integralTotalTop + (energy*exp(-einLam*energy));
                    integralTotalBottom = integralTotalBottom + exp(-einLam*energy);
                end
            end
        else
            for i = 1:numel(spacing)
                x = spacing(i);           
                energy = (1 - cos(k*x/a))/2;
                integralTotalTop = integralTotalTop + (energy*exp(-einLam*energy));
                integralTotalBottom = integralTotalBottom + exp(-einLam*energy);
            end
        end
        msdMean(l) = integralTotalTop/integralTotalBottom;
        integrand(l) = integralTotalTop/integralTotalBottom*einLam;
    end
% 
    f = fit(log(einList'),msdMean'.*einList','cubicinterp');
    x = linspace(log(lambdaMin),log(lambdaMax),1e5);
    FInt = trapz(x,feval(f,x));
    disp(FInt)
    
    disp(dim/2*log(dim/(pi*lambdaMax)) + FInt);
    
    figure(20)
    % clf; hold all
    if dim == 1
        plot(einList,2/dim * integrand,'.-r')
    elseif dim == 2
        plot(einList,2/dim * integrand,'-k','linewidth',1.5)
    elseif dim == 3
        plot(einList,2/dim * integrand,'.-b')
    else
        plot(einList,2/dim * integrand,'.-m')
    end
end
disp(einList(einList>4 & einList<100)')
disp(2/dim * integrand(einList>4&einList<100)')
set(gca,'XScale','log')
xlabel('$\lambda$','interpreter','latex')
ylabel('$\frac{2}{d} \lambda \langle U \rangle$','interpreter','latex')
plotSpecs()
set(gca,'xminortick','off')
set(gca,'Fontsize',20)
xticks([1e-4 1e-2 1e0 1e2 1e4])
set(gca,'Linewidth',1.5)
end