function [pressure,pressureError,k,kerr] = figure2PressureFromCrystal()

dimList = [4:10];

pcut = [NaN NaN 40 40 40 40 40 40 80 100];
nPList = [NaN NaN 2916 5000 3888 5184 17496 6561 39366 81920];

% Set relative base directory
currentFile = mfilename( 'fullpath' );
[pathstr,~,~] = fileparts( currentFile );
basedir = [pathstr,filesep,'..',filesep,'data',filesep,'figure2',filesep,'crystalData'];

for dim = dimList
    if dim < 9
        fitRange =(2:35)';
    else
        fitRange = (20:35)';
    end
    
    nP = nPList(dim);
    for ix = 1:numel(nP)
        % Find all saved data
        nParticles = nP(ix);
        dirlist = dir([basedir,filesep,num2str(dim),filesep,'*']);
        rhoList = zeros(numel(dirlist)-2,1);
        rhoName = cell(numel(dirlist)-2,1);
        place = 1;
        for ii = 1:numel(dirlist)
            if~(strcmp(dirlist(ii).name,'.')||strcmp(dirlist(ii).name,'..'))
                rhoList(place) = str2double(dirlist(ii).name);
                rhoName{place} = dirlist(ii).name;
                place = place+1;
            end
        end
        
        % Set minimum density to plot
        minRho = [0 0 0 0.324 0.206 0.125 0.08 0.0392 0.0225 0.0136]./hyperBallVolume(1:10).*(2.^(1:10));
        if dim == 9
            rhoName = rhoName(rhoList>minRho(dim)&rhoList<8.3);
            rhoList = rhoList(rhoList>minRho(dim)&rhoList<8.3);
        else
            rhoName = rhoName(rhoList>minRho(dim));
            rhoList = rhoList(rhoList>minRho(dim));            
        end
        
        % Initialize arrays
        [rhoList,order] = sort(rhoList);
        rhoName = rhoName(order);
        pressure = NaN(1,numel(rhoList));
        pressureDiff = NaN(1,numel(rhoList));
        pressureError = NaN(2,numel(rhoList));
        rad = NaN(1,numel(rhoList));

        % Set phi_c
        if dim == 6
            phic = pi^3/(48 * sqrt(3));
        elseif dim == 7
            phic = hyperBallVolume(dim)/16;
        elseif dim == 8
            phic = pi^4/(2^4 * 4*3*2);
        elseif dim == 9
            phic = hyperBallVolume(dim) * 2^(-dim/2);
        elseif dim == 10
            phic = 5/128*pi^5/120;
        else
            phic = hyperBallVolume(dim) * 2^(-dim/2 - 1);
        end
        
        rhoc = phic*(2^dim)/hyperBallVolume(dim);
        for i=1:numel(rhoList)
            try
                % Collect g(r) averages
                rho = rhoList(i);
                phi = rho * hyperBallVolume(dim)/(2^dim);
                boxVolume = 2;
                rad(i) = (phi*boxVolume/(nParticles * hyperBallVolume(dim)))^(1/dim);
                saveFile = [basedir,filesep,num2str(dim),filesep,rhoName{i},filesep,num2str(nParticles),filesep,'gofr.dat'];
                gofr = load(saveFile);
                r = gofr(fitRange,1)-1;
                g = gofr(fitRange,2);
                gstd = gofr(fitRange,3);
                if sum(gstd)>0
                    r = r(gstd~=0);
                    g = g(gstd~=0);
                    gstd = gstd(gstd~=0);
                    [res,~] = fit(r,g,'poly1','weights',1./sqrt(gstd));
                else
                    [res,~] = fit(r,g,'poly1');
                end
                
                % Find g(r) at contact
                value = res.p2;
                coplace = 2;
                coeffError = confint(res);
                
                % Calculate pressure from g(r)
                pressureError(1,i) = 2^(dim-1) * phi * (value - coeffError(1,coplace));
                pressureError(2,i) = 2^(dim-1) * phi * (coeffError(2,coplace) - value);
                pressure(i) = 1 + 2^(dim-1) * phi * value;
                thisx = 1-rho;
                pressureDiff(i) = pressure(i) - (1/thisx);
            catch
            end
        end

        % Only use data which is not nans
        r = rhoList';
        x = linspace(min(r(~isnan(pressure))),max(r(~isnan(pressure))),100);
        p = pressure';
        r = r(~isnan(p));
        pe = pressureError(1,~isnan(p));
        p = p(~isnan(p));
        figure(200 + dim)

        try
            % Plot crystal data
            token1 = '.b';
            token2 = '-b';
            errorbar(r*hyperBallVolume(dim) * 2^(-dim),p.*r',pe'.*r',token1,'Linewidth',2,'MarkerSize',15)
            [res,~] = fit(r(p<pcut(dim))',p(p<pcut(dim)),['1/(1-(x/',num2str(rhoc),')^(1/',num2str(dim),')) + a + b*(x/',num2str(rhoc),') + c*(x/',num2str(rhoc),').^2']);        
            plot(x'*hyperBallVolume(dim) * 2^(-dim),feval(res,x).*x',token2,'Linewidth',2)
            con = confint(res);
            disp(res)
        catch
        end
    end
end

% Plot coexistence line
rhof = [NaN NaN 0.4919 0.3031 0.1942 0.1129 0.064799523854637 0.035 0.020405667669440 0.012582130678642]./hyperBallVolume(1:10).*(2.^(1:10));
rhom = [NaN NaN 0.5434 0.3653 0.2484 0.1567 0.096363587290573 0.0558 0.032484284593850 0.021720330682889]./hyperBallVolume(1:10).*(2.^(1:10));
pco = [NaN NaN 11.578 10.807 14.363 16.4 20.43 24.23 38.4 71.9];

uppery = [40 40 40 40 40 40 80 110];
upperx = [0.7 0.55 0.37 0.25 0.15 0.08 0.055 0.028];

for dim = dimList
    figure(200+dim)
    plot([rhof(dim) rhom(dim)] * hyperBallVolume(dim) * 2^(-dim), [pco(dim) pco(dim)],'-k','Linewidth',2)
    xlabel('$\varphi$','interpreter','latex')
    ylabel('$\beta P$','interpreter','latex')
    plotSpecs()
    set(gca,'Linewidth',1.5)
    set(gca,'FontSize',20)
    axis([0 upperx(dim-2) 0 uppery(dim-2)])
    addTextToFigure(200+dim,0.05,0.9,['d=',num2str(dim)],'fontSize',20)
end

end