function [] = figure2PressureFromLiquid()
% This function plots the pressure of the liquid in dimensions 4-10 

% Lists the number of particles that are used in each dimension (d=3 given
% but not used for the liquid)
nParticlesList = [2916 2048 3888 3000 3000 1000 9000 6000];
gofrVersion = 'gofr.dat';
rhoString = {'0.05','0.1','0.15','0.2','0.25','0.3','0.35','0.4','0.45','0.5','0.55','0.6','0.65','0.7','0.75','0.8','0.85','0.9','0.95','1','1.05','1.1','1.15','1.2','1.25','1.3','1.35','1.4','1.45','1.5','1.55','1.6','1.65','1.7','1.75','1.8','1.85','1.9','1.95','2','2.05','2.1','2.15','2.2','2.25','2.3','2.35','2.4','2.45','2.5','2.55','2.6','2.65','2.7'};
rhoList = zeros(1,numel(rhoString));
for i=1:numel(rhoString)
    rhoList(i) = str2double(rhoString{i}); 
end
fitRange =(1:12)';

currentFile = mfilename( 'fullpath' );
[pathstr,~,~] = fileparts( currentFile );
basedir = [pathstr,filesep,'..',filesep,'data',filesep,'figure2',filesep,'liquidData'];

for dim = [4:10]
     nParticles = nParticlesList(dim-2);
     
    % Get values of number density rho
    dirlist = dir([basedir,filesep,num2str(dim),filesep,'*']);
    rhoList = zeros(numel(dirlist)-2,1);
    rhoString = cell(numel(dirlist)-2,1);
    place = 1;
    for ii = 1:numel(dirlist)
        if~(strcmp(dirlist(ii).name,'.')||strcmp(dirlist(ii).name,'..'))
            rhoList(place) = str2double(dirlist(ii).name);
            rhoString{place} = dirlist(ii).name;
            place = place+1;
        end
    end
    pressure = NaN(1,numel(rhoList));
    pressureError = NaN(2,numel(rhoList));
    rad = NaN(1,numel(rhoList));

    for i=1:numel(rhoList)
        try
            rho = rhoList(i);
            % Calculate the packing fraction and radius from rho
            phi = rho * hyperBallVolume(dim) / 2^dim;
            rad(i) = (phi/(nParticles * hyperBallVolume(dim)))^(1/dim);
            % Find the averaged g(r)
            gofr = load([basedir,filesep,num2str(dim),filesep,rhoString{i},filesep,num2str(nParticles),filesep,gofrVersion]);
            r = gofr(:,1)-1;
            g = gofr(:,2);
            thisRange = ~isnan(g)&~isinf(g);
            g = g(thisRange);
            r = r(thisRange);
            g = g(fitRange);
            r = r(fitRange);
            % Find value of g(r) at contact
            [res,~] = fit(r,g,'poly2');
            value = res.p3;
            coplace = 3;
            coeffError = confint(res);
            % Calculate pressure from g(r)
            pressureError(1,i) = 2^(dim-1) * phi * (value - coeffError(1,coplace));
            pressureError(2,i) = 2^(dim-1) * phi * (coeffError(2,coplace) - value);
            pressure(i) = 1 + 2^(dim-1) * phi * value;
        catch
        end
    end
    
    range = pressure < 23;
    
    rho = rhoList(range);
    pressure = pressure(range)'.*rho;
    pressureError = pressureError(:,range)'.*rho;
    rho = [0; rho];
    pressure = [0; pressure];
    pressureError = [0 0; pressureError];
    
    figure(200+dim)
    clf; hold all
    if dim < 5
        y = linspace(0,1.5,1000);
    elseif dim == 5
        y = linspace(min(rho),2,1000);
    elseif dim == 6
        y = linspace(min(rho),2.3,1000);
    elseif dim == 7
        y = linspace(min(rho),2.6,1000);    
    elseif dim == 8
        y = linspace(min(rho),3,1000);
    elseif dim == 9
        y = linspace(min(rho),6,1000);
    elseif dim == 10
        y = linspace(0,7,1000);
    end
    
    % List of pade approximants
    if dim == 3
        Z = ((1 + 1.03544*y + 0.887782*y.^2 + 0.314355*y.^3 + 0.062765*y.^4)./(1 - 1.05896*y + 0.364099*y.^2 - 0.18124*y.^3 + 0.114402*y.^4 - 0.0230037*y.^5)).*y;
    elseif dim ==4
        Z = ((1 + 2.0618*y + 1.5498*y.^2 + 0.1215*y.^3 - 0.3273*y.^4)./(1 - 0.4056*y - 0.5320*y.^2 + 0.4036*y.^3 - 0.0813*y.^4 - 0.0007*y.^5)).*y;
    elseif dim ==5
        Z = ((1 + 5.6285*y + 11.4725*y.^2 + 10.5480*y.^3 + 3.6629*y.^4)./(1 + 2.9966*y + 0.7176*y.^2 - 1.3205*y.^3 + 0.3080*y.^4 - 0.0039*y.^5)).*y;
    elseif dim == 6
        Z = ((1 + 5.6358*y + 11.648*y.^2 + 10.539*y.^3 + 3.4170*y.^4)./(1 + 3.0520*y + 1.4857*y.^2 - 0.8228*y.^3 + 0.0694*y.^4 + 0.0154*y.^5)).*y;
    elseif dim == 7
        Z = ((1 + 5.8810*y + 12.461*y.^2 + 11.291*y.^3 + 3.5776*y.^4)./(1 + 3.5187*y + 2.5736*y.^2 - 0.4615*y.^3 - 0.0638*y.^4 + 0.0261*y.^5)).*y;
    elseif dim == 8
        Z = ((1 + 5.4951*y + 10.767*y.^2 + 8.8764*y.^3 + 2.5049*y.^4)./(1 + 3.4657*y + 2.7671*y.^2 - 0.0664*y.^3 - 0.0859*y.^4 + 0.0163*y.^5)).*y;
    elseif dim == 9
        Z = ((1 + 4.9001*y + 8.5158*y.^2 + 6.1099*y.^3 + 1.5317*y.^4)./(1 + 3.2509*y + 2.6220*y.^2 + 0.1687*y.^3 - 0.0721*y.^4 + 0.0097*y.^5)).*y;
    elseif dim == 10
        Z = ((1 + 5.30618*y + 10.5067*y.^2 + 9.63739*y.^3 + 4.01829*y.^4 + 0.578727*y.^5)./(1 + 4.0311*y + 5.10052*y.^2 + 2.08348*y.^3 + 0.077115*y.^4 - 0.0173544*y.^5 + 0.00129381*y.^6)).*y;
    elseif dim == 11
        Z = ((1 + 2.73556022752334091464*y + 2.63489351164050543614*y.^2 + 1.03878375678973979335*y.^3 + 0.13391037829929309541*y.^4)./(1 + 1.79350828782839091464*y + 0.82345813772020009112*y.^2 + 0.05396943829082206639*y.^3 - 0.00514543438675360267*y.^4 + 0.00028039812008859667*y.^5)).*y;
    elseif dim == 13
        Z = ((1 + 1.30321924886979343581*y + 0.59352076487862641984*y.^2 + 0.10871455065104354578*y.^3 + 0.00630272990617023471*y.^4)./(1 + 0.84790487147815243581*y + 0.18731501198685406265*y.^2 + 0.00724751419672723724*y.^3 - 0.00019310360464809003*y.^4 + 0.00000386632029012702*y.^5)).*y;
    elseif dim == 15
        Z = ((1 + 0.53456694674190725331*y + 0.09943104186276085335*y.^2 + 0.00735894110274800196*y.^3 + 0.00016950110293355809*y.^4)./(1 + 0.34384530633025525331*y + 0.03133519221626758178*y.^2 + 0.00056525316030166359*y.^3 - 0.00000421927966543194 *y.^4 + 0.00000003064369716683*y.^5)).*y;
    elseif dim == 17
        Z = ((1 + 0.21960351994741380895*y + 0.01674788078255364519*y.^2 + 0.00051022453517103890*y.^3 + 0.00000488267307569041*y.^4)./(1 + 0.14911296648884480895*y + 0.00599066942743192235*y.^2 + 0.00005288436507877334*y.^3 - 0.00000011297357633630*y.^4 + 0.00000000032680988932*y.^5)).*y;
    end

% Alternative equations of state which work qualitatively quite well
%     if dim == 8
%         Z = ((0.1153*y.^2 + 0.8625*y + 2.057).*y + 1).*y;
%     end
%     
%     if dim == 9
%         Z = ((0.02572*y.^2 + 0.4777*y + 1.663).*y + 1).*y;
%     end
    
    try
        errorbar(rho*hyperBallVolume(dim) * 2^(-dim),pressure,pressureError(:,1),pressureError(:,2),'.r','MarkerSize',15)
    catch
    end
    plot(y*hyperBallVolume(dim) * 2^(-dim),Z,'-r')
    
end
end
