function [] = figure5a()
% This function performs the einstein integration by pulling the einstein
% crystal simulations, calling an integration function, and plotting Free 
% energy as a function of system size. Instead of using the einstein
% crystal as a reference, however, it uses the periodic potential described
% by Eq. 18 in the paper.


lambdaMin = 1e-6;
lambdaCutoff = 0.99;
dim = 3;
above = 1;

numCellSet = {1,4:2:18,4:2:10,4:2:8,[1536, 2048, 2304, 3456, 5184 23328],[17496],[65536],[39366]};
phiStringList = {'0.4534498410585544','0.5449936404140929','0.3400078716175284','0.20999402376713386','0.1399970496605955','0.08700065938618982','0.14285937829200768','0.09663562292199471'};

% Set relative base directory
currentFile = mfilename( 'fullpath' );
[pathstr,~,~] = fileparts( currentFile );
basedir = [pathstr,filesep,'..',filesep,'data',filesep,'figure5a',filesep];

isPer = 1;
figure(10*dim)
clf; hold all
numCellsList = numCellSet{dim-1};
phiString = phiStringList{dim-1};

cellColor = zeros(numel(numCellsList),3);
cellColor(:,1) = linspace(0,1,numel(numCellsList));
cellColor(:,3) = linspace(1,0,numel(numCellsList));
cellLength = numel(numCellsList);
FexList = NaN(cellLength,1);
for j = 1:cellLength
    try
        figure(dim*10)
        [rho, FexList(j), msdMean,msdIdeal,lambda,msdIdeal2,lambda2,msdEin] = calcExcessFreeEnergy(dim,numCellsList(j),lambdaMin,1000000,phiString,cellColor(j,:),basedir,isPer,above,lambdaCutoff);
    catch
    end
    disp(FexList(j))
end
disp(rho)

N = (numCellsList.^dim)/2;
disp(FexList)

figure(dim*10)
set(gca,'xscale','log');
xlabel('$\lambda$','interpreter','latex')
ylabel('$\frac{2}{d}\lambda \langle U\rangle_\lambda$','interpreter','latex','fontname','helvetica')
plotSpecs()
set(gca,'Linewidth',1.5)
set(gca,'FontSize',24)
set(gca,'xminortick','off')
xticks([1e-4 1e-2 1e0 1e2 1e4 1e6 1e8 1e10])
figure(dim*10)
legendEntry = {};
for i = 1:numel(N)
    legendEntry{i} = ['N=',num2str(N(i))];
end
legend(legendEntry{:},'numcolumns',2,'fontsize',16,'location','northwest')

end
