function [] = hoyCoexistencePlots()
% This function plots the inset of Figure 6 given chemical potential 'mu', 
% pressure 'p', and the number density at freezing 'rhof' for dimensions
% 2-10. Note, the 2d data is taken from Engel et al., PRE (2013) and Clisby
% and McCoy, J. Stat. Phys. (2006)

% Start fit line at d=4
start = 3;

d = 2:10;

%pade diff 0
mu = [12.66 16.082 15.133 17.604 17.697 18.26 17.82 20.2 24.589088098165313];
p = [9.1707 11.578 10.8063 14.3623 16.4 20.43 24.2316 38.4 71.939161848193507];
rhof = [0.7 0.492 0.303 0.194 0.113 0.06656 0.034968 0.0206 0.012559167472574]./hyperBallVolume(d).*2.^d;

muErr = [0.0001 0.004 0.003 0.003 0.003 0.01 0.04 0.1 0.1];
pErr = [0.01 0.003 0.003 0.003 0.004 0.01 0.09 0.1 0.1];

figure(1)
clf; hold all
errorbar(1./d,mu./d,muErr./d,'.b','MarkerSize',25,'Linewidth',2)
errorbar(1./d,p./(d.*rhof),pErr./(d.*rhof),'.r','MarkerSize',25,'Linewidth',2)
xlabel('$1/d$','interpreter','latex')
ylabel('$\widehat{\mu}^\mathrm{coex}$, $\widehat{p}^\mathrm{coex}$','interpreter','latex')
axis([0 0.55 0 7])
plotSpecs()
set(gca,'FontSize',20)
set(gca,'Linewidth',1.5)
x = linspace(1/9,0.5,100);
[res,err] = fit((1./d(start:end))',(p(start:end)./(rhof(start:end).*d(start:end)))','poly1');
plot(x,feval(res,x),'--r','linewidth',2)
[res,err] = fit((1./d(start:end))',(mu(start:end)./d(start:end))','poly1');
plot(x,feval(res,x),'--b','linewidth',2)
legend('$\widehat{\mu}^\mathrm{coex}$','$\widehat{p}^\mathrm{coex}$','location','southeast','interpreter','latex')
pbaspect([1 1.6 1])
xticks([0 0.25 0.5])

end