import monteCarloPCP as mc
import numpy as np
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c
import sys
import os
import glob
import time

# dim = int(sys.argv[1])
# numCells = int(sys.argv[2])
# reducedRhoString = sys.argv[3]
# numAvgs = int(sys.argv[4])
# basedir = sys.argv[5]


dim = 3
numCells = 18
reducedRhoString = '0.97'
numAvgs = 50000
basedir = '/work/pkm8/Data/crystalEOSShortNTest10000001/'

reducedRho = np.float64(reducedRhoString)
if dim==6:
	if numCells < 24:
		numCells = np.array([2,2,2,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 24:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=2)
	else:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=3)
	maxPhi = e6.maxPhi()
	volPoints = 1000000
elif dim==7:
	crystalPos, sides = e7.generateE7Crystal()
	maxPhi = e7.maxPhi()
	volPoints = 1000000
elif dim==8:
	crystalPos, sides = e8.generateE8Crystal()
	maxPhi = e8.maxPhi()
	volPoints = 10000000
else:
	crystalPos, sides = dD.generatedDCrystal(numCells,dim)
	maxPhi = dD.maxPhi(dim)
	volPoints = 1000000

nParticles = crystalPos.shape[0]

saveDir = basedir + '/' + str(dim) + '/' + reducedRhoString + '/' + str(nParticles) + '/'


p = mc.MCPacking(dim,nParticles,maxPhi*reducedRho**dim)
maxRad = (maxPhi/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
p.setBoxSize(sides)
#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1
print(p.getRadii()[0])

if dim>=8:
	saveFreq = 10
	p.setGeometryType(mc.geometryEnum.E8)
elif dim == 7:
	saveFreq = 10
elif dim == 6:
	if numCells <= 24:
		saveFreq = 100
	else:
		saveFreq = 10
else:
	saveFreq = 1000

logBinMin = -3

if nParticles < 1000:
	saveFreq = 10000
	logBinMin = np.log10(1/(nParticles**2))

if dim >= 8:
	cutDist *= 2
elif dim == 7:
	cutDist *= 1.5
elif dim == 6:
	cutDist *= 2
else:
	cutDist *= 3

print(cutDist)
p.setPositions(crystalPos)
p.calcNeighborsCut(cutDist)

# p.calcForceEnergy()
# p.minimizeFIRE(criticalForce = 1e-12)
# p.calcForceEnergy()

minRadius = np.min(p.getRadii())
maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
p.setMCMaxDisplacement(maxDisp)
# Calculate what max displacement should be
p.resetCenterOfMass()
p.setPositions(crystalPos)
numR = 150
g = np.zeros(numR)
r = np.zeros(numR)
numMSD = 3000
msd = np.zeros(numMSD)
p.setRandomSeed(np.random.randint(2**16))
start = len(glob.glob(saveDir + '/*/'))
p.calcNeighborsCut(cutDist);
p.calcForceEnergy()
p.resetDisplacement()
saveNum = start

print(saveDir + '/' + str(saveNum) + '/gofr.dat')
for i in range(int(saveFreq*saveNum)+1,numAvgs+1):
	for j in range(numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			print('resetting at ' + str(j))
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - crystalPos
		# msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		msd[j] = np.mean(np.sum(delta**2,1))
	for j in msd:
		print(j)
	# p.calcNeighborsCut(cutDist)
	thisg,thisr = p.gofr(numR,maxRad = maxDist,minRad = p.getRadii()[0]*2,numVolumePoints = volPoints,logBin = True,logBinMin = logBinMin)
	thisg[np.isnan(thisg)] = 0
	thisg[np.isinf(thisg)] = 0
	r = thisr/(2*p.getRadii()[0])
	g = g+thisg
	if (i%saveFreq == 0):
		os.makedirs(saveDir + '/' + str(saveNum),exist_ok=True)
		mc.fileIO.save2DArray(saveDir + '/' + str(saveNum) + '/gofr.dat',np.stack((thisr/(2*p.getRadii()[0]),g/saveFreq),axis=1))
		g = np.zeros(numR)
		saveNum += 1
