import monteCarloPCP as mc
import numpy as np
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c
import sys
import os
import glob
import time

dim = int(sys.argv[1])
numCells = int(sys.argv[2])
reducedRhoString = sys.argv[3]
numAvgs = int(sys.argv[4])
checkNeighbors = int(sys.argv[5])==1
basedir = sys.argv[6]


# dim = 3
# numCells = 106
# reducedRhoString = '0.95'
# numAvgs = 1000
# checkNeighbors = True
# basedir = '.'

boxVolume = 1
reducedRho = np.float64(reducedRhoString)
if dim==6:
	if numCells < 24:
		numCells = np.array([2,2,2,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 24:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=2)
	else:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=3)
	maxPhi = e6.maxPhi()
	volPoints = 1000000
elif dim==7:
	crystalPos, sides = e7.generateE7Crystal()
	maxPhi = e7.maxPhi()
	volPoints = 1000000
elif dim==8:
	crystalPos, sides = e8.generateE8Crystal()
	maxPhi = e8.maxPhi()
	volPoints = 10000000
elif dim==10:
	crystalPos, sides = p10c.generateP10cCrystal(version = numCells)
	maxPhi = p10c.maxPhi()
	volPoints = 1000000
else:
	volPoints = 1000000
	if numCells < 100:
		crystalPos, sides = dD.generatedDCrystal(numCells,dim)
		maxPhi = dD.maxPhi(dim)
	else:
		crystalPos, sides = dD.generatedDCrystalDBC(numCells-100,dim)
		maxPhi = dD.maxPhi(dim)
		boxVolume = 2


nParticles = crystalPos.shape[0]

saveDir = basedir + '/' + str(dim) + '/' + reducedRhoString + '/' + str(nParticles) + '/'

os.makedirs(saveDir,exist_ok=True)

phi = maxPhi*reducedRho**dim

p = mc.MCPacking(dim,nParticles,phi)
maxRad = (2*maxPhi/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
p.setBoxSize(sides)
#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
# maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
maxDist = (maxRad - p.getRadii()[0]) * 8 + 2*p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1
print(p.getRadii()[0])

if dim>=9:
	saveFreq = 1
else:
	saveFreq = 100

if dim>=8:
	p.setGeometryType(mc.geometryEnum.E8)

if dim==9:
	p.setGeometryType(mc.geometryEnum.Dn)
	boxVolume = 2

if numCells > 100:
	p.setGeometryType(mc.geometryEnum.Dn)
	boxVolume = 2

if dim==10:
	if numCells == 1:
		p.setGeometryType(mc.geometryEnum.E8)
		boxVolume = 1
	else:
		p.setGeometryType(mc.geometryEnum.Dn)
		boxVolume = 2

# 	saveFreq = 10
	
# elif dim == 7:
# 	saveFreq = 10
# elif dim == 6:
# 	if numCells <= 24:
# 		saveFreq = 100
# 	else:
# 		saveFreq = 10
# else:
# 	saveFreq = 10

# logBinMin = -3
logBinMin = np.log10(1/(nParticles))

# if nParticles < 1000:
# 	# saveFreq = 1000
# 	logBinMin = np.log10(1/(nParticles))

if dim == 8:
	cutDist = 1
elif dim >= 9:
	cutDist *= 1
elif dim == 7:
	cutDist *= 1.5
elif dim == 6:
	cutDist *= 2
else:
	cutDist *= 3

print(cutDist)
p.setPositions(crystalPos)
p.calcNeighborsCut(cutDist)
p.setPhi(phi)

# p.calcForceEnergy()
# p.minimizeFIRE(criticalForce = 1e-12)
# p.calcForceEnergy()

minRadius = np.min(p.getRadii())

if os.path.exists(saveDir + '/maxMCDisp.dat'):
	with open(saveDir + '/maxMCDisp.dat', 'r') as file:
		maxDisp = np.float(file.read())
else:
	if dim == 9:
		# maxDisp = (maxRad - p.getRadii()[0])/3
		maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
		p.setMCMaxDisplacementD9Relative(3*maxDisp)
		p.setMCMaxDisplacementCM(maxDisp)
	else:
		maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=40)
	with open(saveDir + '/maxMCDisp.dat', 'w') as file:
		file.write(str(maxDisp) + '\n')

p.setMCMaxDisplacement(maxDisp)
if dim==9:
	p.setMCMaxDisplacementD9Relative(3*maxDisp)

# Calculate what max displacement should be
p.resetCenterOfMass()
p.setPositions(crystalPos)
numR = 75
g = np.zeros(numR)
r = np.zeros(numR)
numMSD = 400
if dim <= 5:
	numMSD = 2000
msd = np.zeros(numMSD)
p.setRandomSeed(np.random.randint(2**16))
start = len(glob.glob(saveDir + '/*/'))
p.calcNeighborsCut(cutDist);
p.calcForceEnergy()
p.resetDisplacement()
saveNum = 0
i = 1
# if maxDist < 3*p.getRadii()[0]:
# 	maxDist = 3*p.getRadii()[0]

print(saveDir + '/' + str(saveNum) + '/gofr.dat')
while saveNum < numAvgs:
	msd = np.zeros(numMSD)
	startingPos = p.getUnwrappedPositions()
	p.resetCenterOfMass()
	for j in range(numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			# print('resetting at ' + str(j))
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - startingPos
		msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		msd[j] = np.mean(np.sum(delta**2,1))
	# 	delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - crystalPos
	# 	msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
	# 	msd[j] = np.mean(np.sum(delta**2,1))
	# for j in msd:
	# 	print(j)
	# p.calcNeighborsCut(cutDist)
	thisg,thisr = p.gofr(numR,maxRad = maxDist,minRad = p.getRadii()[0]*2,numVolumePoints = volPoints,logBin = True,logBinMin = logBinMin)
	thisg[np.isnan(thisg)] = 0
	thisg[np.isinf(thisg)] = 0
	r = thisr/(2*p.getRadii()[0])
	g = g+thisg
	print(i)
	if (i%saveFreq == 0):
		saveNum = len(glob.glob(saveDir + '/*/'))
		os.makedirs(saveDir + '/' + str(saveNum),exist_ok=True)
		mc.fileIO.save2DArray(saveDir + '/' + str(saveNum) + '/gofr.dat',np.stack((thisr/(2*p.getRadii()[0]),g/saveFreq),axis=1))
		g = np.zeros(numR)
		mc.fileIO.saveArray(saveDir + '/' + str(saveNum) + '/msd.dat',msd)
	i = i + 1

