import monteCarloPCP as mc
import numpy as np
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c
import sys
import os
import glob
import time

dim = int(sys.argv[1])
numCells = int(sys.argv[2])
reducedRhoString = sys.argv[3]
numAvgs = int(sys.argv[4])
basedir = sys.argv[5]


# dim = 8
# numCells = 12
# reducedRhoString = '0.99'
# numAvgs = 50000
# basedir = '/work/pkm8/Data/crystalEOSShortNTest10000001/'

reducedRho = np.float64(reducedRhoString)
if dim==6:
	if numCells < 24:
		numCells = np.array([2,2,2,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 24:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=2)
	else:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=3)
	maxPhi = e6.maxPhi()
	volPoints = 1000000
elif dim==7:
	crystalPos, sides = e7.generateE7Crystal()
	maxPhi = e7.maxPhi()
	volPoints = 1000000
elif dim==8:
	crystalPos, sides = e8.generateE8Crystal()
	maxPhi = e8.maxPhi()
	volPoints = 10000000
else:
	crystalPos, sides = dD.generatedDCrystal(numCells,dim)
	maxPhi = dD.maxPhi(dim)
	volPoints = 1000000

nParticles = crystalPos.shape[0]

saveDir = basedir + '/' + str(dim) + '/' + reducedRhoString + '/' + str(nParticles) + '/'

p = mc.MCPacking(dim,nParticles,maxPhi*reducedRho**dim)
maxRad = (maxPhi/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
p.setBoxSize(sides)

#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1
print(p.getRadii()[0])
if dim >= 8:
	p.setGeometryType(mc.geometryEnum.E8)
	cutDist *= 2
elif dim == 7:
	cutDist *= 1.5
elif dim == 6:
	cutDist *= 2
else:
	cutDist *= 3
print(cutDist)

p.setPositions(crystalPos)
p.calcNeighborsCut(cutDist)

minRadius = np.min(p.getRadii())
maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
p.setMCMaxDisplacement(maxDisp)
p.setPositions(crystalPos)
numMSD = 1000
p.setRandomSeed(np.random.randint(2**16))
start = len(glob.glob(saveDir + '/*/'))
p.resetDisplacement()

iteration = start

#Buffer round
numBuffer = 10
bufferIter = 0
while bufferIter < numBuffer:
	p.resetCenterOfMass()	
	msd = np.zeros(numMSD)
	p.calcNeighborsCut(cutDist)
	startingPos = p.getUnwrappedPositions()
	for j in range(numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			print('resetting at ' + str(j))
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
	bufferIter += 1


while iteration < numAvgs:	
	# Calculate what max displacement should be
	p.resetCenterOfMass()	
	msd = np.zeros(numMSD)
	p.calcNeighborsCut(cutDist)
	startingPos = p.getUnwrappedPositions()
	for j in range(numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
			print('resetting at ' + str(j))
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
		p.MCStep()
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - startingPos
		msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		msd[j] = np.mean(np.sum(delta**2,1))
	os.makedirs(saveDir + '/' + str(iteration),exist_ok=True)
	mc.fileIO.saveArray(saveDir + '/' + str(iteration) + '/msd.dat',msd)
	iteration += 1

