import monteCarloPCP as mc
import numpy as np
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c
import sys
import os
import glob
import time

# dim = int(sys.argv[1])
# numCells = int(sys.argv[2])
# rhoString = sys.argv[3]
# numAvgs = int(sys.argv[4])
# checkNeighbors = int(sys.argv[5])==1
# basedir = sys.argv[6]


dim = 6
numCells = 23
rhoString = '1.3'
numAvgs = 5
basedir = 'here4'
checkNeighbors = 1

rho = np.float64(rhoString)
if dim==6:
	if numCells == 19:
		numCells = np.array([3,3,3,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 20:
		numCells = np.array([2,2,2,2,2,2])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 21:
		numCells = np.array([2,2,2,2,2,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 22:
		numCells = np.array([2,2,2,2,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 23:
		numCells = np.array([2,2,2,3,3,3])
		crystalPos, sides = e6.generateE6Crystal(numCells)
	elif numCells == 24:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=2)
	else:
		crystalPos, sides = e6.generateE6Crystal(numCells,version=3)
	maxPhi = e6.maxPhi()
	volPoints = 1000000
elif dim==7:
	crystalPos, sides = e7.generateE7Crystal()
	maxPhi = e7.maxPhi()
	volPoints = 1000000
elif dim==8:
	crystalPos, sides = e8.generateE8Crystal()
	maxPhi = e8.maxPhi()
	volPoints = 1000000
elif dim==10:
	crystalPos, sides = p10c.generateP10cCrystal(version = numCells)
	maxPhi = p10c.maxPhi()
	volPoints = 1000000
else:
	crystalPos, sides = dD.generatedDCrystal(numCells,dim)
	maxPhi = dD.maxPhi(dim)
	volPoints = 1000000

nParticles = crystalPos.shape[0]

saveDir = basedir + '/' + str(dim) + '/' + rhoString + '/' + str(nParticles) + '/'

os.makedirs(saveDir,exist_ok=True)

p = mc.MCPacking(dim,nParticles,1.0)
phi = rho*p.hyperBallCoefficient(dim)/(2**dim)
p.setPhi(phi)

boxVolume = 1

if dim==8:
	p.setGeometryType(mc.geometryEnum.E8)

if dim==9:
	p.setGeometryType(mc.geometryEnum.Dn)
	boxVolume = 2

if dim==10:
	if numCells == 1:
		p.setGeometryType(mc.geometryEnum.E8)
		boxVolume = 1
	else:
		p.setGeometryType(mc.geometryEnum.Dn)
		boxVolume = 2


p.setPhi(phi)

maxRad = (maxPhi * boxVolume/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
p.setBoxSize(sides)
#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1
print(p.getRadii()[0])

if dim < 9:
	saveFreq = 100
else: 
	saveFreq = 10

if dim == 8:
	saveFreq = 10

if dim == 7:
	saveFreq = 1

if dim == 6:
	saveFreq = 10

if dim == 8:
	saveFreq = 1

if dim == 10:
	saveFreq = 1



# elif dim == 7:
# 	saveFreq = 10
# elif dim == 6:
# 	if numCells <= 24:
# 		saveFreq = 100
# 	else:
# 		saveFreq = 10
# else:
# 	saveFreq = 10

# logBinMin = -3
logBinMin = np.log10(1/(nParticles))

# if dim == 6:
# 	logBinMin = np.log10(10/(nParticles))

# if dim == 7:
# 	logBinMin = np.log10(10/(nParticles))

# if nParticles < 1000:
# 	# saveFreq = 1000
# 	logBinMin = np.log10(1/(nParticles))

if dim == 8:
	cutDist = 1
elif dim >= 9:
	cutDist *= 1
elif dim == 7:
	cutDist *= 1.5
elif dim == 6:
	cutDist *= 2
else:
	cutDist *= 3

print(cutDist)
p.setPositions(crystalPos)
if dim < 9:
	p.calcNeighborsCut(cutDist)

if dim ==10:
	p.calcNeighborsCut(cutDist)
# p.calcForceEnergy()
# p.minimizeFIRE(criticalForce = 1e-12)
# p.calcForceEnergy()

minRadius = np.min(p.getRadii())

if os.path.exists(saveDir + '/maxMCDisp.dat'):
	with open(saveDir + '/maxMCDisp.dat', 'r') as file:
		maxDisp = np.float(file.read())
else:
	if dim == 9:
		# maxDisp = (maxRad - p.getRadii()[0])/3
		maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
		p.setMCMaxDisplacementD9Relative(3*maxDisp)
		p.setMCMaxDisplacementCM(maxDisp)
	else:
		maxDisp = p.findMCMaxDisplacement(hiDist = maxDist, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=40)
	with open(saveDir + '/maxMCDisp.dat', 'w') as file:
		file.write(str(maxDisp) + '\n')



p.setMCMaxDisplacement(maxDisp)
# Calculate what max displacement should be
p.resetCenterOfMass()
p.setPositions(crystalPos)
numR = 75
g = np.zeros(numR)
r = np.zeros(numR)
# numMSD = 200
# numMSD = 800
numMSD = 400

# numMSD = 1600


msd = np.zeros(numMSD)
p.setRandomSeed(np.random.randint(2**16))
start = len(glob.glob(saveDir + '/*/'))
print("calculating neighborlist")
p.calcNeighborsCut(cutDist);
p.calcForceEnergy()
p.resetDisplacement()
saveNum = 0
i = 1
neigh = p.getNeighbors()
print("neighbors: " + str(neigh.nnz/(dim*nParticles)))


print(saveDir + '/' + str(saveNum) + '/gofr.dat')
while saveNum < numAvgs:
	for j in range(numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRadius*cutDist):
		# if (checkNeighbors and p.getMaxDisplacement() >= 2*maxRad):
			print('resetting at ' + str(j))
			# p.setPositions(crystalPos)
			p.calcNeighborsCut(cutDist)
			p.resetDisplacement()
			# p.resetCenterOfMass()
			# print("reset")
			j = 0
		p.MCStep()
		print(j)
	# 	delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - crystalPos
	# 	msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
	# 	msd[j] = np.mean(np.sum(delta**2,1))
	# for j in msd:
	# 	print(j)
	# p.calcNeighborsCut(cutDist)
	thisg,thisr = p.gofr(numR,maxRad = maxDist,minRad = p.getRadii()[0]*2,numVolumePoints = volPoints,logBin = True,logBinMin = logBinMin)
	thisg[np.isnan(thisg)] = 0
	thisg[np.isinf(thisg)] = 0
	r = thisr/(2*p.getRadii()[0])
	g = g+thisg
	# print(i)
	if (i%saveFreq == 0):
		saveNum = len(glob.glob(saveDir + '/*/'))
		os.makedirs(saveDir + '/' + str(saveNum),exist_ok=True)
		mc.fileIO.save2DArray(saveDir + '/' + str(saveNum) + '/gofr.dat',np.stack((thisr/(2*p.getRadii()[0]),g/saveFreq),axis=1))
		g = np.zeros(numR)
	i = i + 1

