import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def hyperBallCoefficient(dim):
	return (np.pi**(dim/2.0))/gamma(dim/2.0 + 1)

seed = int(sys.argv[1])
dim = int(sys.argv[2])
numCells = int(sys.argv[3])
phiString = sys.argv[4]
basedir = sys.argv[5]

nParticles = int((numCells**dim)/2)


maxRad = np.sqrt(2.0)*(1.0/numCells)/2.0
maxPhi = hyperBallCoefficient(dim) * nParticles * maxRad**dim
# maxPhi = 4*np.pi/3 * nParticles * maxRad**3
# phi = 0.903*maxPhi
phi = np.float64(phiString)
# phi = np.float64(0.54454)
p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setMonoRadii()
p.setPhi(phi)

# Set up the FCC Crystal
crystalPos = np.zeros((nParticles,dim),dtype=float)
i = 0
for j in range(2*nParticles): 
	testPosition = powerRep(j,numCells,dim)
	if np.sum(testPosition) % 2 == 0:
		crystalPos[i] = testPosition
		i += 1

crystalPos = crystalPos/float(numCells)



# totalMSD = np.zeros(17)

p.setPositions(crystalPos)
p.calcNeighborsCut(1)
p.calcForceEnergy()
p.minimizeFIRE(criticalForce = 1e-12)
sigma = p.getMeanRadius()*2.0
sigSq = sigma**2

minRadius = np.min(p.getRadii())

ij = 0
lambdaList = np.array(['1e-6', '3e-6', '1e-5', '3e-5', '1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6'])
#lambdaList = np.array([2e1, 7e1, 2e2, 7e2, 2e3, 3e3, 7e3])

for einsteinLambdaString in lambdaList:
	saveDir = basedir + '/' + str(dim) + '/' + str(numCells) + '/' + phiString + '/' + str(seed) + '/' + einsteinLambdaString + '/'	
	if not os.path.exists(saveDir + '/radii.dat'):
	# for einsteinLambda in [1e-3, 1e-2, 1e-1, 1e0, 1e1, 1e2, 1e3]:
		einsteinLambda = np.float64(einsteinLambdaString)
		print(einsteinLambda)
		# Calculate crystalPos
		p.setPositions(crystalPos)
		p.calcNeighborsCut(1)
		p.calcForceEnergy()
		p.minimizeFIRE(criticalForce = 1e-12)
		# maxDisp = p.findMCMaxDisplacement(verbose = True)
		# p.setMCMaxDisplacement(maxDisp)	
		p.setEinsteinLambda(einsteinLambda)
		p.setCrystalPositions(crystalPos)
		maxDisp = p.findMCMaxDisplacement(hiDist = 2.0, distTol = 1e-4, neighborCut = 1,verbose = True)
		p.setMCMaxDisplacement(maxDisp)
		# Calculate what max displacement should be
		p.resetCenterOfMass()
		p.setPositions(crystalPos)
		p.calcNeighborsCut(3);
		numMSD = 3000
		msd = np.zeros(numMSD)
		energy = np.zeros(numMSD)
		p.resetDisplacement()
		for j in range(numMSD):
			#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
			if (p.getSumOfMaxDisplacements()/2*minRadius >= 1):
				print('resetting at ' + str(j))
				p.calcNeighborsCut(1)
				p.resetDisplacement()
			p.MCStep()
			delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - p.getCrystalPositions()
			# msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
			msd[j] = np.mean(np.sum(delta**2,1))
		totalMSD = np.mean(msd[int(0.6*numMSD):numMSD])/((2.0*np.mean(p.getRadii()))**2)
		os.makedirs(saveDir,exist_ok=True)
		mc.fileIO.saveArray(saveDir + '/msd.dat',msd)
		with open(saveDir + '/msdPlateau.dat', 'w') as file:
			file.write(str(totalMSD))
		with open(saveDir + '/radii.dat','w') as file:
			file.write(str(np.mean(p.getRadii())))


