import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import Resources.crystalStructures.e6Crystal as e6

seed = int(sys.argv[1])
phiString = sys.argv[2]
numCellSwitch = int(sys.argv[3])
basedir = sys.argv[4]

dim = 6
if numCellSwitch == 0:
	numCells = np.array([1,1,1,1,1,1])
elif numCellSwitch == 1:
	numCells = np.array([1,1,1,2,2,2])
elif numCellSwitch == 2:
	numCells = np.array([2,2,2,2,2,2])
elif numCellSwitch == 3:
	numCells = np.array([2,2,2,2,2,3])
elif numCellSwitch == 4:
	numCells = np.array([2,2,2,2,3,3])
elif numCellSwitch == 5:
	numCells = np.array([2,2,2,3,3,3])

crystalPos,sides = e6.generateE6Crystal(numCells)
nParticles = crystalPos.shape[0]
maxPhi = e6.maxPhi()

phi = np.float64(phiString)
p = mc.MCPacking(dim,nParticles,phi)
p.setBoxSize(sides)
p.setRandomSeed(seed)
p.setMonoRadii()
p.setPhi(phi)


p.setPositions(crystalPos)
p.calcNeighborsCut(1)
p.calcForceEnergy()
sigma = p.getMeanRadius()*2.0
sigSq = sigma**2

minRadius = np.min(p.getRadii())

ij = 0
lambdaList = np.array(['1e-5', '3e-5', '1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6','2e6','3e6','7e6','1e7'])
#lambdaList = np.array([2e1, 7e1, 2e2, 7e2, 2e3, 3e3, 7e3])

for einsteinLambdaString in lambdaList:
	saveDir = basedir + '/' + str(dim) + '/' + str(nParticles) + '/' + phiString + '/' + str(seed) + '/' + einsteinLambdaString + '/'	
	if not os.path.exists(saveDir + '/radii.dat'):
	# for einsteinLambda in [1e-3, 1e-2, 1e-1, 1e0, 1e1, 1e2, 1e3]:
		einsteinLambda = np.float64(einsteinLambdaString)
		print(einsteinLambda)
		# Calculate crystalPos
		p.setPositions(crystalPos)
		p.calcNeighborsCut(1)
		# maxDisp = p.findMCMaxDisplacement(verbose = True)
		# p.setMCMaxDisplacement(maxDisp)	
		p.setEinsteinLambda(einsteinLambda)
		p.setCrystalPositions(crystalPos)
		maxDisp = p.findMCMaxDisplacement(hiDist = 2.0, distTol = 1e-4, neighborCut = 1,verbose = True)
		p.setMCMaxDisplacement(maxDisp)
		# Calculate what max displacement should be
		p.resetCenterOfMass()
		p.setPositions(crystalPos)
		p.calcNeighborsCut(1.2);
		numMSD = 3000
		msd = np.zeros(numMSD)
		energy = np.zeros(numMSD)
		p.resetDisplacement()
		for j in range(numMSD):
			#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
			if (p.getSumOfMaxDisplacements()/2*minRadius >= 1):
				print('resetting at ' + str(j))
				p.calcNeighborsCut(1)
				p.resetDisplacement()
			p.MCStep()
			delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - p.getCrystalPositions()
			# msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
			msd[j] = np.mean(np.sum(delta**2,1))
		totalMSD = np.mean(msd[int(0.6*numMSD):numMSD])/((2.0*np.mean(p.getRadii()))**2)
		os.makedirs(saveDir,exist_ok=True)
		mc.fileIO.saveArray(saveDir + '/msd.dat',msd)
		with open(saveDir + '/msdPlateau.dat', 'w') as file:
			file.write(str(totalMSD))
		with open(saveDir + '/radii.dat','w') as file:
			file.write(str(np.mean(p.getRadii())))



