import monteCarloPCP as mc
import numpy as np
import sys
import os
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c

seed = int(sys.argv[1])
dim = int(sys.argv[2])
numCells = int(sys.argv[3])
xString = sys.argv[4]
checkNeighbors = int(sys.argv[5])==1
basedir = sys.argv[6]

# seed = 32
# dim = 10
# numCells = 0
# xString = '1.00001'
# checkNeighbors=1==1
# basedir = '.'

boxVolume = 1

if dim==6:
	maxPhi = e6.maxPhi()
	numCellSwitch = numCells
	version = 1
	if numCellSwitch == 0:
		numCells = np.array([1,1,1,1,1,1])
	elif numCellSwitch == 1:
		numCells = np.array([1,1,1,2,2,2])
	elif numCellSwitch == 2:
		numCells = np.array([2,2,2,2,2,2])
	elif numCellSwitch == 3:
		numCells = np.array([2,2,2,2,2,3])
	elif numCellSwitch == 4:
		numCells = np.array([2,2,2,2,3,3])
	elif numCellSwitch == 5:
		numCells = np.array([2,2,2,3,3,3])
	elif numCellSwitch == 24:
		version = 2
		numCells = 24
	elif numCellSwitch == 26:
		version = 3
		numCells = 26
	crystalPos, sides = e6.generateE6Crystal(numCells,version=version)
elif dim == 7:
	maxPhi = e7.maxPhi()
	crystalPos, sides = e7.generateE7Crystal(version=numCells)
elif dim == 8:
	maxPhi = e8.maxPhi()
	crystalPos, sides = e8.generateE8Crystal(version=numCells)
	if numCells == 2:
		boxVolume = 2
elif dim ==9:
	crystalPos, sides = dD.generatedDCrystal(numCells,dim)
	maxPhi = dD.maxPhi(dim)
	boxVolume = 2
elif dim==10:
	crystalPos, sides = p10c.generateP10cCrystal(version = numCells)
	maxPhi = p10c.maxPhi()
	boxVolume = 2
else:
	if numCells < 100:
		crystalPos, sides = dD.generatedDCrystal(numCells,dim)
		maxPhi = dD.maxPhi(dim)
	else:
		crystalPos, sides = dD.generatedDCrystalDBC(numCells-100,dim)
		maxPhi = dD.maxPhi(dim)
		boxVolume = 2

nParticles = crystalPos.shape[0]

# maxPhi = 4*np.pi/3 * nParticles * maxRad**3
# phi = 0.903*maxPhi
phi = np.float64(xString)*maxPhi
# phi = np.float64(0.54454)
p = mc.MCPacking(dim,nParticles,phi)
if dim == 8:
	if numCells == 2:
		p.setGeometryType(mc.geometryEnum.Dn)
	elif numCells == 1:
		p.setGeometryType(mc.geometryEnum.E8)


if ((dim < 6 and numCells > 100) or (dim == 9) or (dim == 7 and numCells == 2)):
	p.setGeometryType(mc.geometryEnum.Dn)

if dim == 10:
	p.setGeometryType(mc.geometryEnum.Dn)

maxRad = (maxPhi * boxVolume/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)

p.setBoxSize(sides)
p.setRandomSeed(seed)
p.setMonoRadii()

phiString = str(phi)

# totalMSD = np.zeros(17)

p.setEinsteinType(mc.einsteinEnum.spring)
p.setPhi(phi)
p.setPositions(crystalPos)
p.calcNeighborsCut(0.1)
p.calcForceEnergy()
p.setEinsteinLambda(0)


minRad = np.min(p.getRadii())

ij = 0
lambdaList = np.array(['1e-6', '3e-6', '1e-5', '3e-5', '1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6','1.5e2','5e2','5e1','1.5e1','2.5e2','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9','2e9','3e9','7e9','1e10','2e10','3e10','7e10'])
#lambdaList = np.array(['1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9','2e9','3e9','7e9','1e10'])
# lambdaList = np.array(['1.2626e-4','1.5942e-4','2.0128e-4','2.5413e-4','3.2087e-4','4.0513e-4',
# 	'5.1152e-4','6.4584e-4','8.1544e-4','1.0296e-3','1.2999e-3','1.6413e-3','2.0723e-3',
# 	'2.6165e-3','3.3036e-3','4.1711e-3','5.2665e-3','6.6494e-3','8.3956e-3','1.0600e-2',
# 	'1.3384e-2','1.6898e-2','2.1336e-2','2.6939e-2','3.4013e-2','4.2945e-2','5.4222e-2',
# 	'6.8461e-2','8.6439e-2','1.0914e-1','1.3780e-1','1.7398e-1','2.1967e-1','2.7736e-1',
# 	'3.5019e-1','4.4215e-1','5.5826e-1','7.0486e-1','8.8995e-1','1.1237e0','1.4187e0',
# 	'1.7913e0','2.2617e0','2.8556e0','3.6055e0','4.5523e0','5.7477e0','7.2570e0','9.1627e0',
# 	'1.1569e1','1.4607e1','1.8443e1','2.3286e1','2.9400e1','3.7121e1','4.6869e1','5.9177e1',
# 	'7.4717e1','9.4337e1','1.1911e2','1.5039e2','1.8988e2','2.3974e2','3.0270e2','3.8219e2',
# 	'4.8255e2','6.0927e2','7.6926e2','9.7127e2','1.2263e3','1.5484e3','1.9550e3','2.4683e3',
# 	'3.1165e3','3.9349e3','4.9682e3','6.2729e3','7.9202e3',
# 	'1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', 
# 	'7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', 
# 	'7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5',
# 	'7e5','1e6','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9',
# 	'2e9','3e9','7e9','1e10'])
#lambdaList = np.array([2e1, 7e1, 2e2, 7e2, 2e3, 3e3, 7e3])

einsteinLambdaString = lambdaList[seed]


saveDir = basedir + '/' + str(dim) + '/' + str(numCells) + '/' + phiString + '/' + einsteinLambdaString + '/'	
if dim >= 6:
	saveDir = basedir + '/' + str(dim) + '/' + str(nParticles) + '/' + phiString + '/' + einsteinLambdaString + '/'

maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1

nCut = np.min([cutDist,1.0])

# numMSD = 4000000
numMSD = 400000
numMSD = 30000
numBuffer = 1000
if dim>4:
	numBuffer = 1000
	numMSD = 400000
if dim == 8:
	numBuffer = 1000
	numMSD = 400000
if dim == 9:
	numBuffer = 500
	numMSD = 4000
if dim == 10:
	numBuffer = 500
	numMSD = 4000

msd = np.zeros(numMSD)

runThrough = True
startingPoint = 0

if os.path.exists(saveDir + '/msd.dat'):
	msdLoad = mc.fileIO.loadArray(saveDir + '/msd.dat',dtype=np.float64)
	msdLoad = msdLoad[msdLoad!=0]
	if msd.size <= msdLoad.size:
		np.zeros(msdLoad.size)
		runThrough = False
		startingPoint = msdLoad.size
	for i in range(msdLoad.size):
		msd[i] = msdLoad[i]

os.makedirs(saveDir,exist_ok=True)

if (dim >= 9):
	saveFreq = 200
	backtrack = 20
else:
	saveFreq = 1000
	backtrack = 100

if runThrough:
	einsteinLambda = np.float64(einsteinLambdaString)
	print(einsteinLambda)
	# Calculate crystalPos
	p.setPositions(crystalPos)
	p.calcNeighborsCut(nCut)
	p.calcForceEnergy()
	# maxDisp = p.findMCMaxDisplacement(verbose = True)
	# p.setMCMaxDisplacement(maxDisp)
	p.setEinsteinLambda(einsteinLambda)
	p.setCrystalPositions(p.getUnwrappedPositions())
	if os.path.exists(saveDir + '/maxMCDisp.dat'):
		with open(saveDir + '/maxMCDisp.dat', 'r') as file:
			maxDisp = np.float64(file.read())
	else:
		if dim == 9:
			maxDisp = p.findMCMaxDisplacement(hiDist = (maxRad - p.getRadii()[0])/3, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=1,neighborRecalc=False)
		elif dim == 10:
			maxDisp = p.findMCMaxDisplacement(hiDist = 0.5, distTol = 1e-7, neighborCut = cutDist,verbose = True,numTrials=10)
		else:
			maxDisp = p.findMCMaxDisplacement(hiDist = 0.5, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
		with open(saveDir + '/maxMCDisp.dat', 'w') as file:
			file.write(str(maxDisp))
	p.setMCMaxDisplacement(maxDisp)
	# Calculate what max displacement should be
	p.resetCenterOfMass()
	p.setPositions(crystalPos)
	p.setCrystalPositions(p.getUnwrappedPositions())
	p.calcNeighborsCut(nCut)
	print(p.getNeighbors().nnz)
	p.resetDisplacement()
	p.calcNeighborsCut(nCut)
	energy = np.zeros(numMSD)
	for j in range(numBuffer):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
			# print('resetting at ' + str(j))
			print("resetting")
			# p.setPositions(crystalPos)
			p.calcNeighborsCut(nCut)
			p.resetDisplacement()
			# p.resetCenterOfMass()
			j = 0
		p.MCStep()
	for j in range(startingPoint,numMSD):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
			# print('resetting at ' + str(j))
			print("resetting")
			# p.setPositions(crystalPos)
			p.calcNeighborsCut(nCut)
			p.resetDisplacement()
			# p.resetCenterOfMass()
			j = np.max([startingPoint,j-backtrack])
			for k in range(numBuffer):
				#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
				if (checkNeighbors and p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
					# print('resetting at ' + str(j))
					print("resetting")
					# p.setPositions(crystalPos)
					p.calcNeighborsCut(nCut)
					p.resetDisplacement()
					# p.resetCenterOfMass()
					k = 0
				p.MCStep()
		p.MCStep()
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - crystalPos
		# delta = p.getUnwrappedPositions() - crystalPos
		# msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		msd[j] = np.mean(np.sum(delta**2,1))
		# print("1: " + str(msd[j]))
		# print("2: " + str(p.getEinsteinEnergy()/nParticles*((2.0*np.mean(p.getRadii()))**2)))
		if (j%saveFreq==0 and j>0):
			totalMSD = np.mean(msd[:j])/((2.0*np.mean(p.getRadii()))**2)
			mc.fileIO.saveArray(saveDir + '/msd.dat',msd[:j])
			with open(saveDir + '/msdPlateau.dat', 'w') as file:
				file.write(str(totalMSD))
			with open(saveDir + '/radii.dat','w') as file:
				file.write(str(np.mean(p.getRadii())))
	totalMSD = np.mean(msd)/((2.0*np.mean(p.getRadii()))**2)
	# print(p.getCenterOfMassShift())
	# print(p.getUnwrappedPositions())
	print(p.getPhi())
	mc.fileIO.saveArray(saveDir + '/msd.dat',msd)
	with open(saveDir + '/msdPlateau.dat', 'w') as file:
		file.write(str(totalMSD))
	with open(saveDir + '/radii.dat','w') as file:
		file.write(str(np.mean(p.getRadii())))


