import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import time

# dim = int(sys.argv[1])
# xString = sys.argv[2]
# seed = int(sys.argv[3])
# basedir = sys.argv[4]

# seed = 100
# dim = 3
# xString = '0.736'
# basedir = '/work/pkm8/Data/perTest6/'

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def getOrderParameters(pos,numCells,DBoundaries=False):
	dim = np.size(pos,1)
	nParticles = np.size(pos,0)
	m = np.zeros(dim,dtype=np.float64)
	delta = np.zeros((int(nParticles*(nParticles-1)/2),dim),dtype=np.float64)
	ii = 0
	for i in range(nParticles-1):
		for j in range(i+1,nParticles):
			delta[ii] = p.pbcDisplacement(pos[i],pos[j])
			ii += 1
	if DBoundaries:
		bins = np.linspace(-1,1,100)
	else:
		bins = np.linspace(-0.5,0.5,50)
	for dimIndex in range(dim):
		hist, binEdges = np.histogram(delta[:,dimIndex],bins,density=True)
		binCenters = binEdges[1:] - (binEdges[1]-binEdges[0])/2
		y = np.fft.fft(hist)
		length = np.size(y)
		P2 = np.abs(y)/length
		P1 = P2[:int(length/2+1)]
		P1[1:-1] *= 2
		m[dimIndex] = P1[numCells]
		# Fs = 1/(bins[1]-bins[0])
		# f = Fs * np.arange(length/2)/length
	return m


# dim = 3
# numRows = 4
# basedir = '/work/pkm8/Data/parallelCubeTest7ff2faadsaa/'
# phiString = '0.8'
# crossover = 1
# seed = 107

dim = int(sys.argv[1])
numRows = int(sys.argv[2])
phiString = sys.argv[3]
seed = int(sys.argv[4])
crossover = np.float64(sys.argv[5])
basedir = sys.argv[6]


phi = np.float64(phiString)
ecmcLength = 0.43
nParticles = numRows**dim

p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setParticleType(mc.particleEnum.cube)
p.setPhi(phi)
cryPos = p.getPositions()
for i in range(nParticles):
	cryPos[i] = powerRep(i,numRows,dim)

cryPos = cryPos/numRows

mref = getOrderParameters(cryPos,numRows)

pos = cryPos + p.getRandomVector(nParticles,dim)/10000
p.setPositions(pos)

p.calcNeighborsCut(1)
p.calcEnergy()
p.getEnergy()

latticeSpacing = 1.0/numRows

p.setLatticeSpacing(latticeSpacing)
p.setTargetPressure(1e-2)
p.setMaxVolumeShift(0.0)
# p.setEnsembleType(mc.ensembleEnum.npt)
# p.setMonteCarloType(mc.monteCarloEnum.ecmc)
# p.setECMCLength(ecmcLength)


numSteps = 1000
phiList = np.zeros(numSteps)
tauAlpha = np.zeros(numSteps)
mList = np.zeros((numSteps,dim))

start = 0

#lambdaList = np.array(['1e-6', '3e-6', '1e-5', '3e-5', '1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6'])
lambdaList = np.array(['1.2626e-4','1.5942e-4','2.0128e-4','2.5413e-4','3.2087e-4','4.0513e-4',
	'5.1152e-4','6.4584e-4','8.1544e-4','1.0296e-3','1.2999e-3','1.6413e-3','2.0723e-3',
	'2.6165e-3','3.3036e-3','4.1711e-3','5.2665e-3','6.6494e-3','8.3956e-3','1.0600e-2',
	'1.3384e-2','1.6898e-2','2.1336e-2','2.6939e-2','3.4013e-2','4.2945e-2','5.4222e-2',
	'6.8461e-2','8.6439e-2','1.0914e-1','1.3780e-1','1.7398e-1','2.1967e-1','2.7736e-1',
	'3.5019e-1','4.4215e-1','5.5826e-1','7.0486e-1','8.8995e-1','1.1237e0','1.4187e0',
	'1.7913e0','2.2617e0','2.8556e0','3.6055e0','4.5523e0','5.7477e0','7.2570e0','9.1627e0',
	'1.1569e1','1.4607e1','1.8443e1','2.3286e1','2.9400e1','3.7121e1','4.6869e1','5.9177e1',
	'7.4717e1','9.4337e1','1.1911e2','1.5039e2','1.8988e2','2.3974e2','3.0270e2','3.8219e2',
	'4.8255e2','6.0927e2','7.6926e2','9.7127e2','1.2263e3','1.5484e3','1.9550e3','2.4683e3',
	'3.1165e3','3.9349e3','4.9682e3','6.2729e3','7.9202e3',
	'1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', 
	'7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', 
	'7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5',
	'7e5','1e6','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9',
	'2e9','3e9','7e9','1e10'])
#lambdaList = np.array([2e1, 7e1, 2e2, 7e2, 2e3, 3e3, 7e3])

p.setEinsteinLambda(0)
einsteinLambdaString = lambdaList[seed]
einsteinLambda = np.float64(einsteinLambdaString)


nCut = 1



maxDisp = p.findMCMaxDisplacement(hiDist = 0.5, distTol = 1e-3, neighborCut = nCut,verbose = True,numTrials=1000)
p.setMCMaxDisplacement(maxDisp)

p.setECMCLength(ecmcLength)


p.setNumRows(numRows)

saveFreq = 100

# os.makedirs(saveDir,exist_ok=True)
# UNMUTE




minRad = np.min(p.getRadii())
nCut = 1
p.calcNeighborsCut(nCut)









if einsteinLambda > crossover:
	p.setEinsteinType(mc.einsteinEnum.periodic)
	mcmc = True
else:
	p.setEinsteinType(mc.einsteinEnum.periodicUmbrellaExp)
	mcmc = False


saveDir = basedir + '/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/' + einsteinLambdaString + '/'	


numMSD = 30000
numMSD = 1000
numBuffer = 1000

msd = np.zeros(numMSD)
energy = np.zeros(numMSD)
const = np.zeros(numMSD)

runThrough = True
startingPoint = 0

if os.path.exists(saveDir + '/const.dat'):
	energyLoad = mc.fileIO.loadArray(saveDir + '/energy.dat',dtype=np.float64)
	constLoad = mc.fileIO.loadArray(saveDir + '/const.dat',dtype=np.float64)

os.makedirs(saveDir,exist_ok=True)

saveFreq = 100

einsteinLambda = np.float64(einsteinLambdaString)
p.setEinsteinLambda(einsteinLambda)
p.setCrystalPositions(cryPos)



p.setMCMCBlockedDirection(np.array([False,False,False]))
p.setMonteCarloType(mc.monteCarloEnum.metropolis)
maxDisp = p.findMCMaxDisplacement(hiDist = 10, distTol = 1e-3, verbose = True,numTrials=1000)
p.setMCMaxDisplacement(maxDisp)
maxDispCM = p.findMCMaxDisplacementCM(hiDist = 10, distTol = 1e-3, verbose = True,numTrials=1000)
p.setMCMaxDisplacementCM(maxDispCM)

if (not mcmc):
	p.setMonteCarloType(mc.monteCarloEnum.ecmc)
	p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
	p.setMCMCBlockedDirection(np.array([True,False,False]))
	# p.setMCMCBlockedDirection(np.array([True,True,True]))

# einsteinLambda = np.float64(einsteinLambdaString)
# p.setEinsteinLambda(einsteinLambda)
# p.setCrystalPositions(cryPos)

minRad = np.min(p.getRadii())
# p.calcNeighborsCut(1)
# nCut = 4

energy = np.zeros(numMSD)

j = 0
jj = 0

delta = np.zeros(3)

startPos = p.getUnwrappedPositions()
midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]

while (j < numBuffer):
	if (p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
		# print('resetting at ' + str(j))
		print("resetting")
		p.calcNeighborsCut(nCut)
		p.resetDisplacement()
	if (not mcmc):
		# p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
		p.setECMCLength((0.5 + p.getRandomVector(1,1)[0]))
		test = p.getUnwrappedPositions()
		delta[0] = np.mean(startPos[midx,0] - test[midx,0])
		delta[1] = np.mean(startPos[midy,1] - test[midy,1])
		delta[2] = np.mean(startPos[midz,2] - test[midz,2])
	if mcmc:
		saveCondition = (jj%saveFreq==0 and jj>0)
	else:
		saveCondition = np.any(np.abs(delta) > 1/numRows)
	p.MCStep()
	jj = jj + 1
	if saveCondition:
		j = j + 1

delta = np.zeros(3)

startPos = p.getUnwrappedPositions()
midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]

j = 0
jj = 0

while (j < numMSD):
	if (p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
		# print('resetting at ' + str(j))
		print("resetting")
		p.calcNeighborsCut(nCut)
		p.resetDisplacement()
	if (not mcmc):
		# p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
		p.setECMCLength((0.5 + p.getRandomVector(1,1)[0]))
		test = p.getUnwrappedPositions()
		delta[0] = np.mean(startPos[midx,0] - test[midx,0])
		delta[1] = np.mean(startPos[midy,1] - test[midy,1])
		delta[2] = np.mean(startPos[midz,2] - test[midz,2])
	if mcmc:
		saveCondition = (jj%saveFreq==0 and jj>0)
	else:
		saveCondition = np.any(np.abs(delta) > 1/numRows)
	p.MCStep()
	jj = jj + 1
	if saveCondition:
		energy[j] = p.getEinsteinEnergy()/nParticles
		const[j] = p.getEinsteinWeightedConstant()
		totalMSD = np.mean(energy[:j])/np.mean(const[:j])
		# totalMSD = np.mean(msd[:j])
		mc.fileIO.saveArray(saveDir + '/energy.dat',energy[:j])
		mc.fileIO.saveArray(saveDir + '/const.dat',const[:j])
		with open(saveDir + '/msdPlateau.dat', 'w') as file:
			file.write(str(totalMSD))
		with open(saveDir + '/radii.dat','w') as file:
			file.write(str(np.mean(p.getRadii())))
		if (not mcmc):
			startPos = p.getUnwrappedPositions()
			midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
			midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
			midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]
		j = j+1

print(p.getPhi())
mc.fileIO.saveArray(saveDir + '/energy.dat',energy)
mc.fileIO.saveArray(saveDir + '/const.dat',const)
with open(saveDir + '/msdPlateau.dat', 'w') as file:
	file.write(str(totalMSD))

with open(saveDir + '/radii.dat','w') as file:
	file.write(str(np.mean(p.getRadii())))

