import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c

seed = int(sys.argv[1])
dim = int(sys.argv[2])
numCells = int(sys.argv[3])
phiString = sys.argv[4]
checkNeighbors = int(sys.argv[5])==1
basedir = sys.argv[6]

# seed = 37
# dim = 8
# numCells = 3
# phiString = '0.048'
# basedir = '.'


if dim==6:
	maxPhi = e6.maxPhi()
	numCellSwitch = numCells
	version = 1
	if numCellSwitch == 0:
		numCells = np.array([1,1,1,1,1,1])
	elif numCellSwitch == 1:
		numCells = np.array([1,1,1,2,2,2])
	elif numCellSwitch == 2:
		numCells = np.array([2,2,2,2,2,2])
	elif numCellSwitch == 3:
		numCells = np.array([2,2,2,2,2,3])
	elif numCellSwitch == 4:
		numCells = np.array([2,2,2,2,3,3])
	elif numCellSwitch == 5:
		numCells = np.array([2,2,2,3,3,3])
	elif numCellSwitch == 24:
		version = 2
		numCells = 24
	elif numCellSwitch == 26:
		version = 3
		numCells = 26
	crystalPos, sides = e6.generateE6Crystal(numCells,version=version)
elif dim == 7:
	maxPhi = e7.maxPhi()
	crystalPos, sides = e7.generateE7Crystal()
elif dim == 8:
	maxPhi = e8.maxPhi()
	crystalPos, sides = e8.generateE8Crystal()
else:
	crystalPos, sides = dD.generatedDCrystal(numCells,dim)
	maxPhi = dD.maxPhi(dim)

nParticles = crystalPos.shape[0]

# maxPhi = 4*np.pi/3 * nParticles * maxRad**3
# phi = 0.903*maxPhi
phi = np.float64(phiString)
# phi = np.float64(0.54454)
p = mc.MCPacking(dim,nParticles,phi)
if dim == 8:
	p.setGeometryType(mc.geometryEnum.E8)

maxRad = (maxPhi/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
p.setBoxSize(sides)
p.setRandomSeed(seed)
p.setMonoRadii()

# totalMSD = np.zeros(17)

p.setPositions(crystalPos)
p.calcNeighborsCut(1)
p.calcForceEnergy()

minRadius = np.min(p.getRadii())

ij = 0
#lambdaList = np.array(['1e-6', '3e-6', '1e-5', '3e-5', '1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9'])
lambdaList = np.array(['1e-4', '3e-4', '1e-3', '3e-3', '1e-2', '3e-2', '1e-1', '3e-1', '7e-1', '1e0', '2e0', '3e0', '7e0', '1e1', '2e1', '3e1', '7e1', '1e2', '2e2', '3e2', '7e2', '1e3', '2e3', '3e3', '7e3', '1e4', '2e4', '3e4', '7e4', '1e5', '2e5','3e5','7e5','1e6','2e6','3e6','7e6','1e7','2e7','3e7','7e7','1e8','2e8','3e8','7e8','1e9','2e9','3e9','7e9','1e10'])
#lambdaList = np.array([2e1, 7e1, 2e2, 7e2, 2e3, 3e3, 7e3])

einsteinLambdaString = lambdaList[seed]


saveDir = basedir + '/' + str(dim) + '/' + str(numCells) + '/' + phiString + '/' + einsteinLambdaString + '/'	
if dim >= 6:
	saveDir = basedir + '/' + str(dim) + '/' + str(nParticles) + '/' + phiString + '/' + einsteinLambdaString + '/'

maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = maxDist/(2*p.getRadii()[0]) - 1
cutDist = 2*cutDist

nCut = np.min([cutDist,1.0])

if not os.path.exists(saveDir + '/radii.dat'):
	einsteinLambda = np.float64(einsteinLambdaString)
	print(einsteinLambda)
	# Calculate crystalPos
	p.setPositions(crystalPos)
	p.calcNeighborsCut(1)
	p.calcForceEnergy()
	# maxDisp = p.findMCMaxDisplacement(verbose = True)
	# p.setMCMaxDisplacement(maxDisp)	
	p.setEinsteinLambda(einsteinLambda)
	p.setCrystalPositions(p.getUnwrappedPositions())
	maxDisp = p.findMCMaxDisplacement(hiDist = 2.0, distTol = 1e-4, neighborCut = cutDist,verbose = True,numTrials=10)
	p.setMCMaxDisplacement(maxDisp)
	# Calculate what max displacement should be
	p.resetCenterOfMass()
	p.setPositions(crystalPos)
	p.setCrystalPositions(p.getUnwrappedPositions())
	p.calcNeighborsCut(nCut)
	numMSD = 400000
	p.resetDisplacement()
	numBuffer = 50000
	if dim>4:
		numBuffer = 10000
		numMSD = 400000
	if dim == 8:
		numBuffer = 40000
		numMSD = 4000
	msd = np.zeros(numMSD)
	energy = np.zeros(numMSD)
	for j in range(numBuffer):
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements()/(2*minRadius) >= nCut):
			print('resetting at ' + str(j))
			p.calcNeighborsCut(nCut)
			p.resetDisplacement()
		p.MCStep()
		print(j)
	for j in range(numMSD):
		print(j)
		#Should check how far particles have moved and then recalculate neighbors if it's beyond max mc disp
		if (checkNeighbors and p.getSumOfMaxDisplacements()/(2*minRadius) >= nCut):
			print('resetting at ' + str(j))
			p.calcNeighborsCut(nCut)
			p.resetDisplacement()
		p.MCStep()
		delta = p.getUnwrappedPositions() - p.getCenterOfMassShift() - p.getCrystalPositions()
		# msd[j] = np.mean(np.sum((delta - np.round(delta))**2,1))
		print(delta)
		msd[j] = np.mean(np.sum(delta**2,1))
	totalMSD = np.mean(msd)/((2.0*np.mean(p.getRadii()))**2)
	os.makedirs(saveDir,exist_ok=True)
	mc.fileIO.saveArray(saveDir + '/msd.dat',msd)
	with open(saveDir + '/msdPlateau.dat', 'w') as file:
		file.write(str(totalMSD))
	with open(saveDir + '/radii.dat','w') as file:
		file.write(str(np.mean(p.getRadii())))


