import monteCarloPCP as mc
import numpy as np
import sys
import os
import glob
import time
import Resources.crystalStructures.e8Crystal as e8
import Resources.crystalStructures.e7Crystal as e7
import Resources.crystalStructures.e6Crystal as e6
import Resources.crystalStructures.dDCrystal as dD
import Resources.crystalStructures.p10cCrystal as p10c


dim = int(sys.argv[1])
nParticles = int(sys.argv[2])
rhoString = sys.argv[3]
numR = int(sys.argv[4])
saveFreq = int(sys.argv[5])
numAvgs = int(sys.argv[6])
fullDist = int(sys.argv[7])
useFixedDisp = int(sys.argv[8])
mcOverlapString = sys.argv[9]
basedir = sys.argv[10]

# dim = 5
# nParticles = 600
# rhoString = '1.48'
# numAvgs = 300
# saveFreq = 5
# basedir = '.'
# fullDist = 1
# useFixedDisp = 1
# mcOverlapString = '0.2'
# numR = 300


if dim==6:
	maxPhi = e6.maxPhi()
elif dim==8:
	maxPhi = dD.maxPhi(dim)
elif dim==9:
	maxPhi = dD.maxPhi(dim)
elif dim == 7:
	maxPhi = e7.maxPhi()
else:
	maxPhi = dD.maxPhi(dim)

rho = np.float64(rhoString)
volPoints = 1000000
oldSaveDir = basedir + '/' + mcOverlapString + '/' + str(dim) + '/' + rhoString + '/' + str(nParticles) + '/'
saveDir = basedir + '/' + mcOverlapString + '/' + str(dim) + '/' + rhoString + '/' + str(nParticles) + '/overlap/'
tempDir = saveDir + '/temp/'
oldTempDir = oldSaveDir + '/temp/'


p = mc.MCPacking(dim,nParticles,0.1)

if dim == 3:
	p.setBiRadii()

boxVolume = 2
p.setGeometryType(mc.geometryEnum.Dn)

#if dim==8:
#	p.setGeometryType(mc.geometryEnum.E8)


phi = rho * p.hyperBallCoefficient(dim)/(2.0**dim)
p.setPhi(phi)
maxRad = (maxPhi * boxVolume/(nParticles * p.hyperBallCoefficient(dim)))**(1/dim)
#maxDist = 4*(maxRad - p.getRadii()[0]) + 2*p.getRadii()[0]
maxDist = ((maxRad/p.getRadii()[0] - 1) * 4 + 1) * 2 * p.getRadii()[0]
cutDist = 1


p.setRandomPositions()
p.calcNeighborsCut(cutDist)

p.calcForceEnergy()
p.minimizeFIRE(criticalForce = 1e-12)
p.calcForceEnergy()

p.setMCOverlap(np.float64(mcOverlapString))

maxMCDispFixed = np.array([0,0,0.110,0.060,0.040,0.033,0.027,0.024,0.021,0.019,0.0170,0.016,0.0142,0.013,0.0121,0.011,0.0106,0.008])

print("volume is " + str(p.getVolume()))

minRadius = np.min(p.getRadii())
# Calculate what max displacement should be

os.makedirs(saveDir,exist_ok=True)

if os.path.exists(oldSaveDir + '/tauAlpha.dat'):
	p.load(oldTempDir)
	with open(oldSaveDir + '/neighborCutDistance.dat', 'r') as file:
		neighborCutDistance = np.float(file.read())
	with open(oldSaveDir + '/maxMCDisp.dat', 'r') as file:
		maxMCDisp = np.float(file.read())
else:
	initPos = p.getUnwrappedPositions()
	if useFixedDisp == 1:
		numSteps, tauAlpha, neighborCutDistance, maxMCDisp, msd, overlap = p.thermalizePacking(verbose=True,numTrials=100,numCosts=10,numTau=2,calcMSD=True)
		p.resetCenterOfMass()
		numSteps, tauAlpha, neighborCutDistance, maxMCDisp, msd, overlap = p.thermalizePacking(verbose=True,maxMCDisp=maxMCDispFixed[dim],firstPass=False,numTrials=100,numCosts=30,numTau=3,calcMSD=True)
	else:
		p.resetCenterOfMass()
		numSteps, tauAlpha, neighborCutDistance, maxMCDisp, msd, overlap = p.thermalizePacking(verbose=True,numTrials=100,numCosts=5,numTau=3,calcMSD=True)


rad = p.getRadii()[0]
p.setRandomSeed(np.random.randint(2**16))
start = len(glob.glob(saveDir + '/[0-9]*/'))
p.calcNeighborsCut(neighborCutDistance);
p.calcForceEnergy()
p.resetDisplacement()
saveNum = start

for i in range(saveNum+1,numAvgs+1):
	p.calcNeighborsCut(neighborCutDistance)
	# _,tauAlpha,_,_ = p.thermalizePacking(numTau=3,numTrials=100,numCosts=5,neighborCutDistance=neighborCutDistance,firstPass=False)
	p.resetCenterOfMass()
	initPos = p.getUnwrappedPositions()
	_,tauAlpha,_,_,thisMsd,overlap = p.thermalizePacking(numTau=3,maxMCDisp=maxMCDisp,neighborCutDistance=neighborCutDistance,firstPass=False,calcMSD=True)
	os.makedirs(saveDir + '/' + str(i),exist_ok=True)
	mc.fileIO.saveArray(saveDir + '/' + str(i) + '/overlap.dat',overlap)
	p.save(tempDir)


