import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import time

# dim = int(sys.argv[1])
# xString = sys.argv[2]
# seed = int(sys.argv[3])
# basedir = sys.argv[4]

# seed = 100
# dim = 3
# xString = '0.736'
# basedir = '/work/pkm8/Data/perTest6/'

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def getOrderParameters(pos,numCells,DBoundaries=False):
	dim = np.size(pos,1)
	nParticles = np.size(pos,0)
	m = np.zeros(dim,dtype=np.float64)
	delta = np.zeros((int(nParticles*(nParticles-1)/2),dim),dtype=np.float64)
	ii = 0
	for i in range(nParticles-1):
		for j in range(i+1,nParticles):
			delta[ii] = p.pbcDisplacement(pos[i],pos[j])
			ii += 1
	if DBoundaries:
		bins = np.linspace(-1,1,100)
	else:
		bins = np.linspace(-0.5,0.5,50)
	for dimIndex in range(dim):
		hist, binEdges = np.histogram(delta[:,dimIndex],bins,density=True)
		binCenters = binEdges[1:] - (binEdges[1]-binEdges[0])/2
		y = np.fft.fft(hist)
		length = np.size(y)
		P2 = np.abs(y)/length
		P1 = P2[:int(length/2+1)]
		P1[1:-1] *= 2
		m[dimIndex] = P1[numCells]
		# Fs = 1/(bins[1]-bins[0])
		# f = Fs * np.arange(length/2)/length
	return m

# pressure = np.array([0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1,2,3,4,5,6,7,8,9,10])

start = time.time()

# dim = 3
# numRows = 3
# numTau = 3
# pressureString = '1.422344'
# crystal = 1==0
# basedir = '/work/pkm8/Data/parallelCubeTest72s111/'
# startingPhi = 0.1

dim = int(sys.argv[1])
numRows = int(sys.argv[2])
numTau = int(sys.argv[3])
pressureString = sys.argv[4]
startingPhi = np.float64(sys.argv[5])
crystal = int(sys.argv[6])==1
basedir = sys.argv[7]

if crystal:
	saveDir = basedir + '/Crystal/' + str(dim) + '/' + str(numTau) + '/' + str(numRows) + '/' + pressureString + '/'
else:
	saveDir = basedir + '/Liquid/' + str(dim) + '/' + str(numTau) + '/' + str(numRows) + '/' + pressureString + '/'


pressure = np.float64(pressureString)
ecmcLength = 0.43
seed = 1

nParticles = numRows**dim
phi = startingPhi
p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setParticleType(mc.particleEnum.cube)
crystalPos = p.getPositions()
p.setPhi(phi)
cryPos = p.getPositions()
for i in range(nParticles):
	cryPos[i] = powerRep(i,numRows,dim)

# p.setGeometryType(mc.geometryEnum.Dn)
cryPos = cryPos/numRows

mref = getOrderParameters(cryPos,numRows)

if crystal:
	pos = cryPos + p.getRandomVector(nParticles,dim)/10000
	p.setPositions(pos)
else:
	p.setRandomPositions()
	p.calcNeighborsCut(1)
	p.minimizeFIRE(criticalForce=1e-10)

p.calcNeighborsCut(1)
p.calcEnergy()
p.getEnergy()

p.setTargetPressure(pressure)
p.setMaxVolumeShift(0.1)
p.setEnsembleType(mc.ensembleEnum.npt)
p.setMonteCarloType(mc.monteCarloEnum.ecmc)
p.setECMCLength(ecmcLength)


numSteps = 1000
phiList = np.zeros(numSteps)
tauAlpha = np.zeros(numSteps)
mList = np.zeros((numSteps,dim))
saveFreq = 100

os.makedirs(saveDir,exist_ok=True)
start = 0
# UNMUTE
if os.path.exists(saveDir + '/phi.dat'):
	phiLoad = mc.fileIO.loadArray(saveDir + '/phi.dat',dtype=np.float64)
	mLoad = mc.fileIO.load2DArray(saveDir + '/mList.dat',dtype=np.float64)
	for i in range(len(phiLoad)):
		phiList[i] = phiLoad[i]
		mList[i] = mLoad[i]
	p.load(saveDir + '/temp')
	start = len(phiLoad)
else:
	for i in range(numSteps):
		_,_,_,_,_,_ = p.thermalizePacking(neighborCutDistance=100,numTau=numTau,maxMCDisp=ecmcLength)
		# print(p.getPhi())
		# m = getOrderParameters(p.getPositions(),numRows)/mref
		# print(m)

for i in range(start,numSteps):
	_,tauAlpha[i],_,_,_,_ = p.thermalizePacking(neighborCutDistance=100,numTau=numTau,maxMCDisp=ecmcLength)
	phiList[i] = p.getPhi()
	# print(p.getPhi())
	mList[i] = getOrderParameters(p.getPositions(),numRows)/mref
	# print(mList[i])
	if (i%saveFreq==0 and i>0):
		mc.fileIO.saveArray(saveDir + '/phi.dat',phiList[:i])
		mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:i])
		p.save(saveDir + '/temp')

finish = time.time()
print(finish-start)


# if os.path.exists(saveDir + '/tauAlpha.dat'):
# 	with open(saveDir + '/tauAlpha.dat', 'r') as file:
# 		tauAlpha = int(file.read())
# 	with open(saveDir + '/meanPhi.dat', 'r') as file:
# 		meanPhi = np.float64(file.read())
# else:
# 	_,tauAlpha,_,_ = p.thermalizePacking(neighborCutDistance=100,numTau=10,maxMCDisp=ecmcLength)
# 	with open(saveDir + '/meanPhi.dat', 'w') as file:
# 		_ = file.write(str(np.mean(phiList)))
# 	with open(saveDir + '/tauAlpha.dat', 'w') as file:
# 		_ = file.write(str(tauAlpha))

# for i in range(1000):
# 	print(mList[i])


