import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import time

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def getOrderParameters(pos,numCells,DBoundaries=False):
	dim = np.size(pos,1)
	nParticles = np.size(pos,0)
	m = np.zeros(dim,dtype=np.float64)
	delta = np.zeros((int(nParticles*(nParticles-1)/2),dim),dtype=np.float64)
	ii = 0
	for i in range(nParticles-1):
		for j in range(i+1,nParticles):
			delta[ii] = p.pbcDisplacement(pos[i],pos[j])
			ii += 1
	if DBoundaries:
		bins = np.linspace(-1,1,100)
	else:
		bins = np.linspace(-0.5,0.5,50)
	for dimIndex in range(dim):
		hist, binEdges = np.histogram(delta[:,dimIndex],bins,density=True)
		binCenters = binEdges[1:] - (binEdges[1]-binEdges[0])/2
		y = np.fft.fft(hist)
		length = np.size(y)
		P2 = np.abs(y)/length
		P1 = P2[:int(length/2+1)]
		P1[1:-1] *= 2
		m[dimIndex] = P1[numCells]
		# Fs = 1/(bins[1]-bins[0])
		# f = Fs * np.arange(length/2)/length
	return m

# dim = 2
# numRows = 5
# phiString = '0.2'
# vacancyFractionString = '0.0'
# crystal = 1==1
# numSteps = 1000
# basedir = '/work/pkm8/Data/parallelCubeTest72s111/'


dim = int(sys.argv[1])
numRows = int(sys.argv[2])
phiString = sys.argv[3]
vacancyFractionString = sys.argv[4]
crystal = int(sys.argv[5])==1
numSteps = int(sys.argv[6])
basedir = sys.argv[7]

if crystal:
	saveDir = basedir + '/Crystal/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/' + vacancyFractionString + '/'
else:
	saveDir = basedir + '/Liquid/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/' + vacancyFractionString + '/'


ecmcLength = 0.43
seed = np.random.randint(2**16)
neighborCut = 1

nParticles = numRows**dim
phi = np.float64(phiString)
p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setParticleType(mc.particleEnum.cube)
crystalPos = p.getPositions()
p.setPhi(phi)
cryPos = p.getPositions()
for i in range(nParticles):
	cryPos[i] = powerRep(i,numRows,dim)

# p.setGeometryType(mc.geometryEnum.Dn)
cryPos = cryPos/numRows

mref = getOrderParameters(cryPos,numRows)

cryPos = cryPos + p.getRandomVector(nParticles,dim)/10000

if crystal:
	pos = cryPos
	p.setPositions(pos)
else:
	p.setRandomPositions()
	p.calcNeighborsCut(1)
	p.minimizeFIRE(criticalForce=1e-10)

p.removeParticles(numParticles=int(np.floor(np.float64(vacancyFractionString)*p.getNumParticles())))
p.setPhi(phi)

cryPos = p.getPositions()

p.calcNeighborsCut(1)
p.calcEnergy()
p.getEnergy()
p.minimizeFIRE(criticalForce=1e-10)

p.setNumRows(numRows)

print(p.getRadii()[0])
# p.setTargetPressure(pressure)

# p.setEnsembleType(mc.ensembleEnum.npt)
p.setMonteCarloType(mc.monteCarloEnum.ecmc)
p.setECMCLength(ecmcLength)

p.setMCMCBlockedDirection(np.array([True,True,True]))
maxVolumeShift = p.findMaxVolumeStep(hiDist=1,verbose=True)

# maxVolumeShift = 0.0

p.setMaxVolumeShift(maxVolumeShift)


neighborRecalc = ~np.all(p.getMCMCBlockedDirection())

numTrials = 50
numTrue = np.zeros(numTrials)
pressure = np.zeros(numTrials)
boxSize = np.zeros((numTrials,2))
minRadius = np.min(p.getRadii())


# for i in range(numTrials):
# 	p.setECMCLength(0.5 + p.getRandomVector(1,1)[0])
# 	p.MCStep()
# 	if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
# 		p.calcNeighborsCut(neighborCut)
# 		p.resetDisplacement()
# 		p.calcEnergy()
# 	numTrue[i] = np.sum(p.getBoxSize()**2)
# 	p.MCStepAspectRatio(0)
# 	if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
# 		p.calcNeighborsCut(neighborCut)
# 		p.resetDisplacement()
# 		p.calcEnergy()
# 	boxSize[i] = p.getBoxSize()
# 	pressure[i] = p.getECMCPressure()

# print(numTrue)
# print(np.mean(numTrue))
# print(p.getPositions())
# print(p.getRadii()[0])
# print("nParticles is " + str(p.getNumParticles()))
# print("Mean pressure is " + str(np.mean(pressure)))
# print("Mean box size is " + str(np.mean(boxSize,0)))

saveFreq = 100

os.makedirs(saveDir,exist_ok=True)
# UNMUTE

numBuffer = 1000
pressure = np.zeros(numSteps)
mList = np.zeros((numSteps,dim))
boxSizeSet = np.zeros((numSteps,dim))

# randBuff = 0.5 + p.getRandomVector(1,numBuffer)
# randLength = 0.5 + p.getRandomVector(1,numSteps)


# pressure = np.zeros(5)

# p1 = p.getPositions()
# p.setECMCLength(randLength[0,0])
# p.MCStep()
# p2 = p.getPositions()
# pressure[0] = p.getECMCPressure()
# p.setECMCLength(randLength[0,1])
# p.MCStep()
# p3 = p.getPositions()
# pressure[1] = p.getECMCPressure()
# p.setECMCLength(randLength[0,2])
# p.MCStep()
# p4 = p.getPositions()
# pressure[2] = p.getECMCPressure()
# p.setECMCLength(randLength[0,3])
# p.MCStep()
# p5 = p.getPositions()
# pressure[3] = p.getECMCPressure()
# p.setECMCLength(randLength[0,4])
# p.MCStep()
# p6 = p.getPositions()
# pressure[4] = p.getECMCPressure()


# pressure = np.zeros(1000)
# for i in range(1000):
# 	p.setECMCLength(randLength[0,i])
# 	p.MCStep()
# 	pressure[i] = p.getECMCPressure()


meanRand = 1.0 - 2.0 * minRadius * p.getNumParticles()**(1/dim)
stdRand = meanRand/2.0

randBuff = meanRand + stdRand*p.getRandomGaussianVector(1,numBuffer)
randLength = meanRand + stdRand*p.getRandomGaussianVector(1,numSteps)

while len(np.where(randBuff<0)[1])>0:
	randBuff[0,np.where(randBuff<0)[1]] = meanRand + stdRand*p.getRandomGaussianVector(1,len(np.where(randBuff<0)[1]))

while len(np.where(randLength<0)[1])>0:
	randLength[0,np.where(randLength<0)[1]] = meanRand + stdRand*p.getRandomGaussianVector(1,len(np.where(randLength<0)[1]))


randLength[0,np.where(randLength<0)[1]] = 0.0

print(meanRand)
print(stdRand)


start = 0
if os.path.exists(saveDir + '/pressure.dat'):
	pLoad = mc.fileIO.loadArray(saveDir + '/pressure.dat',dtype=np.float64)
	mLoad = mc.fileIO.load2DArray(saveDir + '/mList.dat',dtype=np.float64)
	boxSizeLoad = mc.fileIO.load2DArray(saveDir + '/boxSize.dat',dtype=np.float64)
	for i in range(len(pLoad)):
		pressure[i] = pLoad[i]
		mList[i] = mLoad[i]
		boxSizeSet[i] = boxSizeLoad[i]
	p.load(saveDir + '/temp')
	start = len(pLoad)
	print("loading")
else:
	for i in range(numBuffer):
		if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
			p.calcNeighborsCut(neighborCut)
			p.resetDisplacement()
			p.calcEnergy()
		print(i)
		print(randBuff[0,i])
		p.setECMCLength(randBuff[0,i])
		p.MCStep()
		print(p.getECMCPressure())
		if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
			p.calcNeighborsCut(neighborCut)
			p.resetDisplacement()
			p.calcEnergy()
		p.MCStepAspectRatio(0)
	p.save(saveDir + '/temp')

print("On to main round")

for i in range(start,numSteps):
	p.setECMCLength(randLength[0,i])
	if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
		p.calcNeighborsCut(neighborCut)
		p.resetDisplacement()
		p.calcEnergy()
	p.MCStep()
	if (neighborRecalc and p.getSumOfMaxDisplacements() >= 2*neighborCut*minRadius):
		p.calcNeighborsCut(neighborCut)
		p.resetDisplacement()
		p.calcEnergy()
	p.MCStepAspectRatio(0)
	pressure[i] = p.getECMCPressure()
	mList[i] = getOrderParameters(p.getPositions(),numRows)/mref
	boxSizeSet[i] = p.getBoxSize()
	if (i%saveFreq==0 and i>0):
		print("saving " + str(i))
		mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:i])
		mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:i])
		mc.fileIO.save2DArray(saveDir + '/boxSize.dat',boxSizeSet[:i])
		p.save(saveDir + '/temp')

print("end set")
mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:i])
mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:i])
mc.fileIO.save2DArray(saveDir + '/boxSize.dat',boxSizeSet[:i])
