import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import time

# dim = int(sys.argv[1])
# xString = sys.argv[2]
# seed = int(sys.argv[3])
# basedir = sys.argv[4]

# seed = 100
# dim = 3
# xString = '0.736'
# basedir = '/work/pkm8/Data/perTest6/'

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def getOrderParameters(pos,numCells,DBoundaries=False):
	dim = np.size(pos,1)
	nParticles = np.size(pos,0)
	m = np.zeros(dim,dtype=np.float64)
	delta = np.zeros((int(nParticles*(nParticles-1)/2),dim),dtype=np.float64)
	ii = 0
	for i in range(nParticles-1):
		for j in range(i+1,nParticles):
			delta[ii] = p.pbcDisplacement(pos[i],pos[j])
			ii += 1
	if DBoundaries:
		bins = np.linspace(-1,1,100)
	else:
		bins = np.linspace(-0.5,0.5,50)
	for dimIndex in range(dim):
		hist, binEdges = np.histogram(delta[:,dimIndex],bins,density=True)
		binCenters = binEdges[1:] - (binEdges[1]-binEdges[0])/2
		y = np.fft.fft(hist)
		length = np.size(y)
		P2 = np.abs(y)/length
		P1 = P2[:int(length/2+1)]
		P1[1:-1] *= 2
		m[dimIndex] = P1[numCells]
		# Fs = 1/(bins[1]-bins[0])
		# f = Fs * np.arange(length/2)/length
	return m



# dim = 3
# numRows = 4
# phiString = '0.72'
# numSteps = 1000
# crystal = 1==1
# basedir = '/work/pkm8/Data/parallelCubeTest72s111/'

dim = int(sys.argv[1])
numRows = int(sys.argv[2])
phiString = sys.argv[3]
numSteps = int(sys.argv[4])
crystal = int(sys.argv[5])==1
basedir = sys.argv[6]

if crystal:
	saveDir = basedir + '/Crystal/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/'
else:
	saveDir = basedir + '/Liquid/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/'


ecmcLength = 0.43
seed = 1

mcmc = False

nParticles = numRows**dim
phi = np.float64(phiString)
p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setParticleType(mc.particleEnum.cube)
crystalPos = p.getPositions()
p.setPhi(phi)
cryPos = p.getPositions()
for i in range(nParticles):
	cryPos[i] = powerRep(i,numRows,dim)

# p.setGeometryType(mc.geometryEnum.Dn)
cryPos = cryPos/numRows

mref = getOrderParameters(cryPos,numRows)
p.setNumRows(numRows)

if crystal:
	pos = cryPos + p.getRandomVector(nParticles,dim)/10000
	p.setPositions(pos)
else:
	p.setRandomPositions()
	p.calcNeighborsCut(1)
	p.minimizeFIRE(criticalForce=1e-10)



saveFreq = 100
nCut = 1
p.calcNeighborsCut(nCut)
p.calcEnergy()
p.getEnergy()

print(p.getRadii()[0])
# p.setTargetPressure(pressure)
# p.setMaxVolumeShift(0.1)
# p.setEnsembleType(mc.ensembleEnum.npt)

p.setMCMCBlockedDirection(np.array([False,False,False]))
p.setMonteCarloType(mc.monteCarloEnum.metropolis)
maxDisp = p.findMCMaxDisplacement(hiDist = 10, distTol = 1e-3, verbose = True,numTrials=1000)
p.setMCMaxDisplacement(maxDisp)

if (not mcmc):
	p.setMonteCarloType(mc.monteCarloEnum.ecmc)
	p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
	p.setMCMCBlockedDirection(np.array([True,False,False]))
	# p.setMCMCBlockedDirection(np.array([True,True,True]))




os.makedirs(saveDir,exist_ok=True)
# UNMUTE

numBuffer = 1000
pressure = np.zeros(numSteps,dtype=np.float64)
pressureStd = np.zeros(numSteps,dtype=np.float64)
tauAlpha = np.zeros(numSteps,dtype=int)
mList = np.zeros((numSteps,dim),dtype=np.float64)


minRad = np.min(p.getRadii())

delta = np.zeros(3)

startPos = p.getUnwrappedPositions()
midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]

j = 0
jj = 0

start = 0
if os.path.exists(saveDir + '/pressure.dat'):
	pLoad = mc.fileIO.loadArray(saveDir + '/pressure.dat',dtype=np.float64)
	mLoad = mc.fileIO.load2DArray(saveDir + '/mList.dat',dtype=np.float64)
	for i in range(len(pLoad)):
		pressure[i] = pLoad[i]
		mList[i] = mLoad[i]
	p.load(saveDir + '/temp')
	start = len(pLoad)
	print("loading")
else:
	startPos = p.getUnwrappedPositions()
	midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
	midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
	midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]
	while (j < numBuffer):
		if (p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
			# print('resetting at ' + str(j))
			# print("resetting")
			p.calcNeighborsCut(nCut)
			p.resetDisplacement()
		if (not mcmc):
			p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
			# p.setECMCLength((0.5 + p.getRandomVector(1,1)[0]))
			test = p.getUnwrappedPositions()
			delta[0] = np.mean(startPos[midx,0] - test[midx,0])
			delta[1] = np.mean(startPos[midy,1] - test[midy,1])
			delta[2] = np.mean(startPos[midz,2] - test[midz,2])
		p.MCStep()
		if mcmc:
			saveCondition = (jj%saveFreq==0 and jj>0)
		else:
			saveCondition = np.any(np.abs(delta) > 1/numRows)
		jj = jj + 1
		if saveCondition:
			j = j + 1

startPos = p.getUnwrappedPositions()
midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]

j = start
jj = 0
tau = 0

thisPressureList = np.array([])

while (j < numSteps):
	if (p.getSumOfMaxDisplacements() >= 2*minRad*nCut):
		# print('resetting at ' + str(j))
		# print("resetting")
		p.calcNeighborsCut(nCut)
		p.resetDisplacement()
	if (not mcmc):
		p.setECMCLength(4*maxDisp*p.getRadii()[0]*(0.5 + p.getRandomVector(1,1)[0]))
		# p.setECMCLength((0.5 + p.getRandomVector(1,1)[0]))
		test = p.getUnwrappedPositions()
		delta[0] = np.mean(startPos[midx,0] - test[midx,0])
		delta[1] = np.mean(startPos[midy,1] - test[midy,1])
		delta[2] = np.mean(startPos[midz,2] - test[midz,2])
	if mcmc:
		saveCondition = (jj%saveFreq==0 and jj>0)
	else:
		saveCondition = np.any(np.abs(delta) > 1/numRows)
	p.MCStep()
	jj = jj + 1
	tau = tau + 1
	thisPressureList = np.append(thisPressureList,p.getECMCPressure())
	if saveCondition:
		pressure[j] = np.mean(thisPressureList)
		pressureStd[j] = np.std(thisPressureList)
		mList[j] = getOrderParameters(p.getPositions(),numRows)/mref
		tauAlpha[j] = tau
		mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:j])
		mc.fileIO.saveArray(saveDir + '/pressureStd.dat',pressureStd[:j])
		mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:j])
		mc.fileIO.saveArray(saveDir + '/tauAlpha.dat',tauAlpha[:j])
		p.save(saveDir + '/temp')
		if (not mcmc):
			startPos = p.getUnwrappedPositions()
			midx = np.where(np.round(p.getPositions()*numRows)[:,0]==3)[0]
			midy = np.where(np.round(p.getPositions()*numRows)[:,1]==3)[0]
			midz = np.where(np.round(p.getPositions()*numRows)[:,2]==3)[0]
		j = j+1
		tau = 0
		print(j)
		thisPressureList = np.array([])

mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:j])
mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:j])


