import monteCarloPCP as mc
import numpy as np
import sys
import os
from scipy.special import gamma
import time

# dim = int(sys.argv[1])
# xString = sys.argv[2]
# seed = int(sys.argv[3])
# basedir = sys.argv[4]

# seed = 100
# dim = 3
# xString = '0.736'
# basedir = '/work/pkm8/Data/perTest6/'

def powerRep(number,base,digits):
	# Turns a number into a representation in a base, cutoff such that the limit is base**digits
	# First, create an empty representation
	rep = [0]*digits
	for thisDigit in range(digits,0,-1):
		rep[thisDigit-1], number = divmod(number, base**(thisDigit-1))
	rep.reverse()
	return rep

def getOrderParameters(pos,numCells,DBoundaries=False):
	dim = np.size(pos,1)
	nParticles = np.size(pos,0)
	m = np.zeros(dim,dtype=np.float64)
	delta = np.zeros((int(nParticles*(nParticles-1)/2),dim),dtype=np.float64)
	ii = 0
	for i in range(nParticles-1):
		for j in range(i+1,nParticles):
			delta[ii] = p.pbcDisplacement(pos[i],pos[j])
			ii += 1
	if DBoundaries:
		bins = np.linspace(-1,1,100)
	else:
		bins = np.linspace(-0.5,0.5,50)
	for dimIndex in range(dim):
		hist, binEdges = np.histogram(delta[:,dimIndex],bins,density=True)
		binCenters = binEdges[1:] - (binEdges[1]-binEdges[0])/2
		y = np.fft.fft(hist)
		length = np.size(y)
		P2 = np.abs(y)/length
		P1 = P2[:int(length/2+1)]
		P1[1:-1] *= 2
		m[dimIndex] = P1[numCells]
		# Fs = 1/(bins[1]-bins[0])
		# f = Fs * np.arange(length/2)/length
	return m



dim = 2
numRows = 3
phiString = '0.77'
crystal = 1==0
basedir = '/work/pkm8/Data/parallelCubeTest72s11/'

# dim = int(sys.argv[1])
# numRows = int(sys.argv[2])
# phiString = sys.argv[3]
# crystal = int(sys.argv[4])==1
# basedir = sys.argv[5]

if crystal:
	saveDir = basedir + '/Crystal/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/'
else:
	saveDir = basedir + '/Liquid/' + str(dim) + '/' + str(numRows) + '/' + phiString + '/'


ecmcLength = 0.43
seed = 1

nParticles = numRows**dim
nParticles = 131
phi = np.float64(phiString)
p = mc.MCPacking(dim,nParticles,phi)
p.setRandomSeed(seed)
p.setParticleType(mc.particleEnum.cube)
crystalPos = p.getPositions()
p.setPhi(phi)
cryPos = p.getPositions()
for i in range(nParticles):
	cryPos[i] = powerRep(i,numRows,dim)

# p.setGeometryType(mc.geometryEnum.Dn)
cryPos = cryPos/numRows

mref = getOrderParameters(cryPos,numRows)

if crystal:
	pos = cryPos + p.getRandomVector(nParticles,dim)/10000
	p.setPositions(pos)
else:
	p.setRandomPositions()
	p.calcNeighborsCut(1)
	p.minimizeFIRE(criticalForce=1e-10)

p.calcNeighborsCut(1)
p.calcEnergy()
p.getEnergy()

print(p.getRadii()[0])
nSteps, tauAlpha, nCut, maxMCDisp, msd, overlap = p.thermalizePacking(verbose=True,numTau = 10)


minRadius = np.min(p.getRadii())
for i in range(10 * tauAlpha):
	p.MCStep()
	if (p.getSumOfMaxDisplacements() >= 2*minRadius*nCut):
		p.calcNeighborsCut(nCut)
		p.resetDisplacement()

r1 = p.getRadii()
p1 = p.getPositions()
p.removeParticles(np.array([0]))
r2 = p.getRadii()
p2 = p.getPositions()

# p.setTargetPressure(pressure)
# p.setMaxVolumeShift(0.1)
# p.setEnsembleType(mc.ensembleEnum.npt)
# p.setMonteCarloType(mc.monteCarloEnum.ecmc)
# p.setECMCLength(ecmcLength)


saveFreq = 100

os.makedirs(saveDir,exist_ok=True)
# UNMUTE

numBuffer = 100000
numSteps = 5000
pressure = np.zeros(numSteps)
mList = np.zeros((numSteps,dim))

randBuff = 0.5 + p.getRandomVector(1,numBuffer)
randLength = 0.5 + p.getRandomVector(1,numSteps)


# pressure = np.zeros(5)

# p1 = p.getPositions()
# p.setECMCLength(randLength[0,0])
# p.MCStep()
# p2 = p.getPositions()
# pressure[0] = p.getECMCPressure()
# p.setECMCLength(randLength[0,1])
# p.MCStep()
# p3 = p.getPositions()
# pressure[1] = p.getECMCPressure()
# p.setECMCLength(randLength[0,2])
# p.MCStep()
# p4 = p.getPositions()
# pressure[2] = p.getECMCPressure()
# p.setECMCLength(randLength[0,3])
# p.MCStep()
# p5 = p.getPositions()
# pressure[3] = p.getECMCPressure()
# p.setECMCLength(randLength[0,4])
# p.MCStep()
# p6 = p.getPositions()
# pressure[4] = p.getECMCPressure()


# pressure = np.zeros(1000)
# for i in range(1000):
# 	p.setECMCLength(randLength[0,i])
# 	p.MCStep()
# 	pressure[i] = p.getECMCPressure()


# randBuff = meanRand + stdRand*p.getRandomGaussianVector(1,numBuffer)
# randLength = meanRand + stdRand*p.getRandomGaussianVector(1,numSteps)



start = 0
if os.path.exists(saveDir + '/pressure.dat'):
	pLoad = mc.fileIO.loadArray(saveDir + '/pressure.dat',dtype=np.float64)
	mLoad = mc.fileIO.load2DArray(saveDir + '/mList.dat',dtype=np.float64)
	for i in range(len(pLoad)):
		pressure[i] = pLoad[i]
		mList[i] = mLoad[i]
	p.load(saveDir + '/temp')
	start = len(pLoad)
	print("loading")
else:
	for i in range(numSteps):
		p.setECMCLength(randBuff[0,i])
		# print(randBuff[0,i])
		p.MCStep()

for i in range(start,numSteps):
	p.setECMCLength(randLength[0,i])
	p.MCStep()
	pressure[i] = p.getECMCPressure()
	mList[i] = getOrderParameters(p.getPositions(),numRows)/mref
	if (i%saveFreq==0 and i>0):
		mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:i])
		mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:i])
		p.save(saveDir + '/temp')

mc.fileIO.saveArray(saveDir + '/pressure.dat',pressure[:i])
mc.fileIO.save2DArray(saveDir + '/mList.dat',mList[:i])


