import numpy as np
import monteCarloPCP as mc
import os.path

def generateE6LatticePoint(lVec):
	e6 = np.array([
	[0,0,0,np.sqrt(3),0,0],
	[0,0,0,0,np.sqrt(3),0],
	[1,1,1,0,0,0],
	[1,-1/2,-1/2,0,-np.sqrt(3)/2,-np.sqrt(3)/2],
	[-1/2,1,-1/2,-np.sqrt(3)/2,0,-np.sqrt(3)/2],
	[1/2,1/2,1/2,np.sqrt(3)/2,np.sqrt(3)/2,np.sqrt(3)/2]
	])
	return lVec[0]*e6[0,:] + lVec[1]*e6[1,:] + lVec[2]*e6[2,:] + lVec[3]*e6[3,:] + lVec[4]*e6[4,:] + lVec[5]*e6[5,:]

def generateE6Crystal(numCellArray,version=1):
	if version == 2:
		my_path = os.path.abspath(os.path.dirname(__file__))
		path = os.path.join(my_path, "e6N2048Pos.dat")
		print(path)
		cryPos = mc.fileIO.load2DArray(path,dtype=float)
		sides = np.ones(6)
		sides[5] = np.sqrt(3)/2
		sides /= np.product(sides)**(1/6)
	elif version == 3:
		my_path = os.path.abspath(os.path.dirname(__file__))
		path = os.path.join(my_path, "e6N23328Pos.dat")
		print(path)
		cryPos = mc.fileIO.load2DArray(path,dtype=float)
		sides = np.ones(6)
		sides[5] = np.sqrt(3)/2
		sides /= np.product(sides)**(1/6)
	else:
		cryPos = np.zeros((24*np.product(numCellArray),6))
		unitBound = np.array([3,3,3,np.sqrt(3),np.sqrt(3),np.sqrt(3)])
		latticeVectors = np.array([
			[-1, -1, 0, 1, 1, 3],
			[-1, 0, 0, 1, 0, 2],
			[0, -1, 0, 0, 1, 2],
			[0, 0, 0, 0, 0, 0],
			[0, 0, 0, 0, 0, 1],
			[0, 0, 0, 0, 1, 1],
			[0, 0, 0, 1, 0, 1],
			[0, 0, 0, 1, 1, 2],
			[0, 0, 1, -1, -1, -1],
			[0, 0, 1, -1, 0, 0],
			[0, 0, 1, 0, -1, 0],
			[0, 0, 1, 0, 0, 0],
			[0, 0, 1, 0, 0, 1],
			[0, 0, 1, 0, 1, 1],
			[0, 0, 1, 1, 0, 1],
			[0, 0, 1, 1, 1, 2],
			[0, 0, 2, -1, -1, -1],
			[0, 0, 2, -1, 0, 0],
			[0, 0, 2, 0, -1, 0],
			[0, 0, 2, 0, 0, 0],
			[0, 0, 2, 0, 0, 1],
			[0, 1, 2, 0, -1, -1],
			[1, 0, 2, -1, 0, -1],
			[1, 1, 2, -1, -1, -2]
			])
		place = 0
		for i0 in range(numCellArray[0]):
			for i1 in range(numCellArray[1]):
				for i2 in range(numCellArray[2]):
					for i3 in range(numCellArray[3]):
						for i4 in range(numCellArray[4]):
							for i5 in range(numCellArray[5]):
								offset = np.array([i0,i1,i2,i3,i4,i5])*unitBound
								for i in range(24):
									cryPos[place,:] = generateE6LatticePoint(latticeVectors[i]) + offset
									place = place + 1
		unitVolume = np.product(unitBound*numCellArray)**(1/6)
		for i in range(6):
			cryPos[:,i] *= 1/(unitVolume)
		sides = unitBound * numCellArray
	return cryPos, sides

def maxPhi():
	return np.pi**3/(48 * np.sqrt(3))